@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -39,7 +39,11 @@
 #include "filejob.h"
 #include "freespaceinfo.h"
 #include "icons.h"
-#include <QtCore/QFile>
+#include "config.h"
+#ifdef ENABLE_ONLINE_SERVICES
+#include "onlineservicesmodel.h"
+#endif
+#include <QFile>
 
 static int iCount=0;
 
@@ -81,6 +85,50 @@ ActionDialog::~ActionDialog()
     iCount--;
 }
 
+void ActionDialog::controlInfoLabel(Device *dev)
+{
+    DeviceOptions opts;
+    if (Device::AudioCd==dev->devType()) {
+        opts.load(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
+    } else {
+        opts=dev->options();
+    }
+
+    if (opts.transcoderCodec.isEmpty()) {
+        codecLabel->setVisible(false);
+        codec->setVisible(false);
+    } else {
+        codecLabel->setVisible(true);
+        codec->setVisible(true);
+        Encoders::Encoder encoder=Encoders::getEncoder(opts.transcoderCodec);
+        if (Encoders::getEncoder(opts.transcoderCodec).codec.isEmpty()) {
+            codec->setText(i18n("<b>INVALID</b>"));
+        } else if (encoder.values.count()>1) {
+            int settingIndex=0;
+            bool increase=encoder.values.at(0).value<encoder.values.at(1).value;
+            int index=0;
+            foreach (const Encoders::Setting &s, encoder.values) {
+                if ((increase && s.value>opts.transcoderValue) || (!increase && s.value<opts.transcoderValue)) {
+                    break;
+                } else {
+                    settingIndex=index;
+                }
+                index++;
+            }
+            codec->setText(QString("%1 (%2)").arg(encoder.name).arg(encoder.values.at(settingIndex).descr)+
+                           (Device::AudioCd!=dev->devType() && opts.transcoderWhenDifferent ? QLatin1String("")+i18n("<i>(When different)</i>") : QString()));
+        } else {
+            codec->setText(encoder.name+
+                           (Device::AudioCd!=dev->devType() && opts.transcoderWhenDifferent ? QLatin1String("")+i18n("<i>(When different)</i>") : QString()));
+        }
+    }
+}
+
+void ActionDialog::controlInfoLabel()
+{
+    controlInfoLabel(getDevice(sourceUdi.isEmpty() ? destUdi : sourceUdi));
+}
+
 void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QList<Song> &songs)
 {
     init(srcUdi, dstUdi, songs, Copy);
@@ -91,9 +139,12 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
         return;
     }
 
+    controlInfoLabel(dev);
+
     // check space...
     haveVariousArtists=false;
     qint64 spaceRequired=0;
+    totalTime=0;
     foreach (const Song &s, songsToAction) {
         if (s.size>0) {
             spaceRequired+=s.size;
@@ -101,11 +152,13 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
         if (!haveVariousArtists && s.isVariousArtists()) {
             haveVariousArtists=true;
         }
+        totalTime+=s.time;
     }
 
     qint64 spaceAvailable=0;
     double usedCapacity=0.0;
     QString capacityString;
+    //bool isFromOnline=sourceUdi.startsWith(OnlineServicesModel::constUdiPrefix);
 
     if (sourceUdi.isEmpty()) { // If source UDI is empty, then we are copying from MPD->device, so need device free space.
         spaceAvailable=dev->freeSpace();
@@ -127,7 +180,11 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
     if (enoughSpace || (sourceUdi.isEmpty() && Encoders::getAvailable().count())) {
         QString mpdCfgName=MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name);
         overwrite->setChecked(Settings::self()->overwriteSongs());
-        sourceLabel->setText(QLatin1String("<b>")+(sourceUdi.isEmpty() ? i18n("Local Music Library") : dev->data())+QLatin1String("</b>"));
+        sourceLabel->setText(QLatin1String("<b>")+(sourceUdi.isEmpty()
+                                ? i18n("Local Music Library")
+                                : Device::AudioCd==dev->devType()
+                                    ? i18n("Audio CD")
+                                    : dev->data())+QLatin1String("</b>"));
         destinationLabel->setText(QLatin1String("<b>")+(destUdi.isEmpty() ? i18n("Local Music Library") : dev->data())+QLatin1String("</b>"));
         namingOptions.load(mpdCfgName, true);
         setPage(PAGE_START);
@@ -138,7 +195,10 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
         bool destIsDev=sourceUdi.isEmpty();
         mpdConfigured=DeviceOptions::isConfigured(mpdCfgName, true);
         configureDestLabel->setVisible((destIsDev && !dev->isConfigured()) || (!destIsDev && !mpdConfigured));
-        configureSourceLabel->setVisible((!destIsDev && !dev->isConfigured()) || (destIsDev && !mpdConfigured));
+        //configureSourceButton->setVisible(!isFromOnline);
+        //configureSourceLabel->setVisible(!isFromOnline && ((!destIsDev && !dev->isConfigured()) || (destIsDev && !mpdConfigured)));
+        configureSourceButton->setVisible(false);
+        configureSourceLabel->setVisible(false);
         show();
         if (!enoughSpace) {
             MessageBox::information(this, i18n("There is insufficient space left on the destination device.\n"
@@ -159,6 +219,8 @@ void ActionDialog::copy(const QString &srcUdi, const QString &dstUdi, const QLis
 void ActionDialog::remove(const QString &udi, const QList<Song> &songs)
 {
     init(udi, QString(), songs, Remove);
+    codecLabel->setVisible(false);
+    codec->setVisible(false);
     QString baseDir;
 
     if (udi.isEmpty()) {
@@ -198,6 +260,10 @@ void ActionDialog::init(const QString &srcUdi, const QString &dstUdi, const QLis
     performingAction=false;
     paused=false;
     actionedSongs.clear();
+    skippedSongs.clear();
+    actionedTime=0;
+    timeTaken=0;
+    currentPercent=0;
     currentDev=0;
     count=0;
 }
@@ -209,16 +275,17 @@ void ActionDialog::slotButtonClicked(int button)
         switch (button) {
         case Ok:
             if (haveVariousArtists &&
-                ((configureDestLabel->isVisible() &&
+                ((configureDestLabel->isVisible() && sourceUdi.isEmpty() && // Only warn if copying FROM library
                   MessageBox::No==MessageBox::warningYesNo(this, i18n("<p>You have not configured the destination device.<br/>"
                                                                       "Continue with the default settings?</p>"))) ||
-                 (configureSourceLabel->isVisible() &&
+                 (configureSourceLabel->isVisible() && !sourceUdi.isEmpty() && // Only warn if copying TO library
                   MessageBox::No==MessageBox::warningYesNo(this, i18n("<p>You have not configured the source device.<br/>"
                                                                       "Continue with the default settings?</p>"))))) {
                 return;
             }
             Settings::self()->saveOverwriteSongs(overwrite->isChecked());
             setPage(PAGE_PROGRESS);
+            timer.start();
             doNext();
             break;
         case Cancel:
@@ -233,9 +300,13 @@ void ActionDialog::slotButtonClicked(int button)
         }
         break;
     case PAGE_SKIP:
+        timeTaken+=timer.elapsed();
         setPage(PAGE_PROGRESS);
         switch(button) {
         case User1:
+            skippedSongs.append(currentSong);
+            totalTime-=currentSong.time;
+            timer.restart();
             incProgress();
             doNext();
             break;
@@ -263,7 +334,23 @@ void ActionDialog::slotButtonClicked(int button)
     case PAGE_PROGRESS:
         paused=true;
         if (MessageBox::Yes==MessageBox::questionYesNo(this, i18n("Are you sure you wish to cancel?"))) {
-            refreshLibrary();
+            Device *dev=0;
+            if(Copy==mode) {
+                dev=getDevice(sourceUdi.isEmpty() ? destUdi : sourceUdi, false);
+            } else if (!sourceUdi.isEmpty()) { // Must be a delete...
+                dev=getDevice(sourceUdi, false);
+            }
+
+            if (dev) {
+                if (Close==button) { // Close is only enabled when saving cache...
+                    disconnect(dev, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
+                } else {
+                    dev->abortJob();
+                }
+            }
+            if (Close!=button) {
+                refreshLibrary();
+            }
             reject();
             // Need to call this - if not, when dialog is closed by window X control, it is not deleted!!!!
             Dialog::slotButtonClicked(button);
@@ -275,9 +362,19 @@ void ActionDialog::slotButtonClicked(int button)
     }
 }
 
-Device * ActionDialog::getDevice(const QString &udi)
+Device * ActionDialog::getDevice(const QString &udi, bool logErrors)
 {
+    #ifdef ENABLE_ONLINE_SERVICES
+    if (udi.startsWith(OnlineServicesModel::constUdiPrefix)) {
+        return OnlineServicesModel::self()->device(udi);
+    }
+    #endif
     Device *dev=DevicesModel::self()->device(udi);
+
+    if (!logErrors) {
+        return dev;
+    }
+
     QString error;
     if (!dev) {
         error=i18n("Device has been removed!");
@@ -304,6 +401,7 @@ Device * ActionDialog::getDevice(const QString &udi)
 
 void ActionDialog::doNext()
 {
+    currentPercent=0;
     if (songsToAction.count()) {
         currentSong=origCurrentSong=songsToAction.takeFirst();
         if(Copy==mode) {
@@ -348,7 +446,7 @@ void ActionDialog::doNext()
                 }
             }
         }
-        progressLabel->setText(formatSong(currentSong));
+        progressLabel->setText(formatSong(currentSong, false, true));
     } else if (Remove==mode && dirsToClean.count()) {
         Device *dev=sourceUdi.isEmpty() ? 0 : DevicesModel::self()->device(sourceUdi);
         if (sourceUdi.isEmpty() || dev) {
@@ -361,32 +459,35 @@ void ActionDialog::doNext()
         }
         dirsToClean.clear();
     } else {
-        refreshLibrary();
-        emit completed();
-        accept();
+        if (!refreshLibrary()) {
+            emit completed();
+            accept();
+        }
     }
 }
 
 void ActionDialog::actionStatus(int status, bool copiedCover)
 {
     int origStatus=status;
+    bool wasSkip=false;
     if (Device::Ok!=status && Device::NotConnected!=status && autoSkip) {
+        skippedSongs.append(currentSong);
+        totalTime-=currentSong.time;
+        wasSkip=true;
         status=Device::Ok;
     }
     switch (status) {
     case Device::Ok:
         performingAction=false;
         if (Device::Ok==origStatus) {
-            actionedSongs.append(currentSong);
+            if (!wasSkip) {
+                actionedSongs.append(currentSong);
+                actionedTime+=currentSong.time;
+            }
             if (copiedCover) {
                 copiedCovers.insert(Utils::getDir(destFile));
             }
         }
-//         if (1==actionedSongs.count() && ( (Copy==mode && !sourceUdi.isEmpty()) ||
-//                                           (Remove==mode && sourceUdi.isEmpty()) ) ) {
-//             // Cache is now out of date, so need to remove!
-//             MusicLibraryModel::self()->removeCache();
-//         }
         if (!paused) {
             incProgress();
             doNext();
@@ -423,6 +524,29 @@ void ActionDialog::actionStatus(int status, bool copiedCover)
     case Device::FailedToCreateTempFile:
         setPage(PAGE_ERROR, i18n("Failed to create temporary file.<br/>(Required for transcoding to MTP devices.)<hr/>%1").arg(formatSong(currentSong)));
         break;
+    case Device::ReadFailed:
+        setPage(PAGE_SKIP, i18n("Failed to read source file.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::WriteFailed:
+        setPage(PAGE_SKIP, i18n("Failed to write to destination file.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::NoSpace:
+        setPage(PAGE_SKIP, i18n("No space left on device.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::FailedToUpdateTags:
+        setPage(PAGE_SKIP, i18n("Failed to update metadata.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::TooManyRedirects:
+        setPage(PAGE_SKIP, i18n("Failed to download track - too many redirects encountered.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::DownloadFailed:
+        setPage(PAGE_SKIP, i18n("Failed to download track.<br/><br/<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::FailedToLockDevice:
+        setPage(PAGE_ERROR, i18n("Failed to lock device.<hr/>%1").arg(formatSong(currentSong)));
+        break;
+    case Device::Cancelled:
+        break;
     default:
         break;
     }
@@ -443,34 +567,49 @@ void ActionDialog::configureSource()
 void ActionDialog::configure(const QString &udi)
 {
     if (udi.isEmpty()) {
+        Device *dev=DevicesModel::self()->device(sourceUdi);
+        bool isCd=false;
+        if (dev) {
+            isCd=Device::AudioCd==dev->devType();
+        }
         DevicePropertiesDialog *dlg=new DevicePropertiesDialog(this);
-        connect(dlg, SIGNAL(updatedSettings(const QString &, const QString &, const DeviceOptions &)),
-                SLOT(saveProperties(const QString &, const QString &, const DeviceOptions &)));
+        connect(dlg, SIGNAL(updatedSettings(const QString &, const DeviceOptions &)), SLOT(saveProperties(const QString &, const DeviceOptions &)));
         if (!mpdConfigured) {
             connect(dlg, SIGNAL(cancelled()), SLOT(saveProperties()));
         }
         dlg->setCaption(i18n("Local Music Library Properties"));
-        dlg->show(MPDConnection::self()->getDetails().dir, QString(), namingOptions, DevicePropertiesWidget::Prop_Basic);
+        dlg->show(MPDConnection::self()->getDetails().dir, namingOptions, DevicePropertiesWidget::Prop_Basic|(isCd ? DevicePropertiesWidget::Prop_Encoder : 0));
+        connect(dlg, SIGNAL(destroyed()), SLOT(controlInfoLabel()));
     } else {
         Device *dev=DevicesModel::self()->device(udi);
         if (dev) {
             dev->configure(this);
+            connect(dev, SIGNAL(configurationChanged()), SLOT(controlInfoLabel()));
         }
     }
 }
 
-void ActionDialog::saveProperties(const QString &path, const QString &coverFile, const DeviceOptions &opts)
+void ActionDialog::saveProperties(const QString &path, const DeviceOptions &opts)
 {
     Q_UNUSED(path)
-    Q_UNUSED(coverFile)
+    Device *dev=DevicesModel::self()->device(sourceUdi);
+    bool isCd=false;
+    if (dev) {
+        isCd=Device::AudioCd==dev->devType();
+    }
     namingOptions=opts;
-    namingOptions.save(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
+    namingOptions.save(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true, isCd);
     mpdConfigured=true;
 }
 
 void ActionDialog::saveProperties()
 {
-    namingOptions.save(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true);
+    Device *dev=DevicesModel::self()->device(sourceUdi);
+    bool isCd=false;
+    if (dev) {
+        isCd=Device::AudioCd==dev->devType();
+    }
+    namingOptions.save(MPDConnectionDetails::configGroupName(MPDConnection::self()->getDetails().name), true, isCd);
     mpdConfigured=true;
 }
 
@@ -510,34 +649,47 @@ void ActionDialog::setPage(int page, const QString &msg)
     }
 }
 
-QString ActionDialog::formatSong(const Song &s, bool showFiles)
+QString ActionDialog::formatSong(const Song &s, bool showFiles, bool showTime)
 {
-    return showFiles
-            ? Copy==mode
-                ? i18n("<table>"
-                       "<tr><td align=\"right\">Artist:</td><td>%1</td></tr>"
-                       "<tr><td align=\"right\">Album:</td><td>%2</td></tr>"
-                       "<tr><td align=\"right\">Track:</td><td>%3</td></tr>"
-                       "<tr><td align=\"right\">Source file:</td><td>%4</td></tr>"
-                       "<tr><td align=\"right\">Destination file:</td><td>%5</td></tr>"
-                       "</table>").arg(s.albumArtist()).arg(s.album)
-                       .arg(s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()) && !Song::isVariousArtists(s.artist))).arg(s.file).arg(destFile)
-                : i18n("<table>"
-                       "<tr><td align=\"right\">Artist:</td><td>%1</td></tr>"
-                       "<tr><td align=\"right\">Album:</td><td>%2</td></tr>"
-                       "<tr><td align=\"right\">Track:</td><td>%3</td></tr>"
-                       "<tr><td align=\"right\">File:</td><td>%4</td></tr>"
-                       "</table>").arg(s.albumArtist()).arg(s.album)
-                       .arg(s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()) && !Song::isVariousArtists(s.artist))).arg(s.file)
-            : i18n("<table>"
-                   "<tr><td align=\"right\">Artist:</td><td>%1</td></tr>"
-                   "<tr><td align=\"right\">Album:</td><td>%2</td></tr>"
-                   "<tr><td align=\"right\">Track:</td><td>%3</td></tr>"
-                   "</table>").arg(s.albumArtist()).arg(s.album)
-                       .arg(s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()) && !Song::isVariousArtists(s.artist)));
+    QString str("<table>");
+    str+=i18n("<tr><td align=\"right\">Artist:</td><td>%1</td></tr>"
+              "<tr><td align=\"right\">Album:</td><td>%2</td></tr>"
+              "<tr><td align=\"right\">Track:</td><td>%3</td></tr>")
+              .arg(s.albumArtist())
+              .arg(s.album)
+              .arg(s.trackAndTitleStr(Song::isVariousArtists(s.albumArtist()) && !Song::isVariousArtists(s.artist)));
+
+    if (showFiles) {
+        if (Copy==mode) {
+            str+=i18n("<tr><td align=\"right\">Source file:</td><td>%1</td></tr>"
+                      "<tr><td align=\"right\">Destination file:</td><td>%2</td></tr>")
+                      .arg(DevicesModel::fixDevicePath(s.file))
+                      .arg(DevicesModel::fixDevicePath(destFile));
+
+        } else {
+            str+=i18n("<tr><td align=\"right\">File:</td><td>%1</td></tr>")
+                      .arg(DevicesModel::fixDevicePath(s.file));
+        }
+    }
+
+    if (showTime) {
+        QString estimate=i18n("Unknown");
+
+        double taken=timeTaken+timer.elapsed();
+        if (taken>5.0) {
+            double percent=(actionedTime+(currentPercent*currentSong.time))/totalTime;
+            quint64 timeRemaining=((taken/percent)-taken)/1000.0;
+            estimate=i18nc("time (EStimated)", "%1 (Estimated)").arg(Song::formattedTime(timeRemaining>0 ? timeRemaining : 0));
+        }
+
+        str+=i18n("<tr><i><td align=\"right\"><i>Time remaining:</i></td><td><i>%5</i></td></i></tr>")
+                .arg(estimate);
+    }
+
+    return str+"</table>";
 }
 
-void ActionDialog::refreshLibrary()
+bool ActionDialog::refreshLibrary()
 {
     actionLabel->stopAnimation();
     if (!actionedSongs.isEmpty()) {
@@ -551,9 +703,14 @@ void ActionDialog::refreshLibrary()
 
             if (dev) {
                 dev->saveCache();
+                progressLabel->setText(i18n("Saving cache"));
+                connect(dev, SIGNAL(cacheSaved()), this, SLOT(cacheSaved()));
+                setButtons(Close);
+                return true;
             }
         }
     }
+    return false;
 }
 
 void ActionDialog::removeSong(const Song &s)
@@ -571,8 +728,8 @@ void ActionDialog::removeSong(const Song &s)
 void ActionDialog::removeSongResult(int status)
 {
     FileJob::finished(sender());
-    if (FileJob::StatusOk!=status) {
-        actionStatus(Device::Failed);
+    if (Device::Ok!=status) {
+        actionStatus(status);
     } else {
         MusicLibraryModel::self()->removeSongFromList(currentSong);
         DirViewModel::self()->removeFileFromList(currentSong.file);
@@ -591,12 +748,16 @@ void ActionDialog::cleanDirs()
 void ActionDialog::cleanDirsResult(int status)
 {
     FileJob::finished(sender());
-    actionStatus(FileJob::StatusOk==status ? Device::Ok : Device::Failed);
+    actionStatus(status);
 }
 
 void ActionDialog::jobPercent(int percent)
 {
     progressBar->setValue((100*count)+percent);
+    currentPercent=percent/100.00;
+    if (PAGE_PROGRESS==stack->currentIndex()) {
+        progressLabel->setText(formatSong(currentSong, false, true));
+    }
 }
 
 void ActionDialog::incProgress()
@@ -609,3 +770,9 @@ void ActionDialog::incProgress()
         progressBar->setValue(val<=progressBar->maximum() ? val : progressBar->maximum());
     }
 }
+
+void ActionDialog::cacheSaved()
+{
+    emit completed();
+    accept();
+}
