@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -30,17 +30,21 @@
 #include "networkaccessmanager.h"
 #include "settings.h"
 #include "localize.h"
+#include "actioncollection.h"
 #ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KStandardDirs>
 #include <KDE/KGlobal>
 K_GLOBAL_STATIC(Dynamic, instance)
 #endif
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
-#include <QtCore/QProcess>
-#include <QtCore/QTimer>
-#include <QtGui/QIcon>
+#include <QDir>
+#include <QFile>
+#include <QTextStream>
+#include <QProcess>
+#include <QTimer>
+#include <QIcon>
+#include <QUrl>
+#if QT_VERSION >= 0x050000
+#include <QUrlQuery>
+#endif
 #include <signal.h>
 
 static const QString constDir=QLatin1String("dynamic");
@@ -88,13 +92,14 @@ Dynamic::Dynamic()
     : timer(0)
     , statusTime(1)
     , currentJob(0)
-    , manager(0)
 {
     loadLocal();
     connect(this, SIGNAL(clear()), MPDConnection::self(), SLOT(clear()));
     connect(MPDConnection::self(), SIGNAL(dynamicUrl(const QString &)), this, SLOT(dynamicUrlChanged(const QString &)));
     connect(MPDConnection::self(), SIGNAL(statusUpdated(const MPDStatusValues &)), this, SLOT(checkRemoteHelper()));
-    QTimer::singleShot(0, this, SLOT(checkHelper()));
+    QTimer::singleShot(500, this, SLOT(checkHelper()));
+    startAction = ActionCollection::get()->createAction("startdynamic", i18n("Start Dynamic Playlist"), "media-playback-start");
+    stopAction = ActionCollection::get()->createAction("stopdynamic", i18n("Stop Dynamic Mode"), "process-stop");
 }
 
 QVariant Dynamic::headerData(int, Qt::Orientation, int) const
@@ -139,7 +144,7 @@ QVariant Dynamic::data(const QModelIndex &index, int role) const
     case Qt::ToolTipRole:
         return entryList.at(index.row()).name;
     case Qt::DecorationRole:
-        return IS_ACTIVE(entryList.at(index.row()).name) ? QIcon::fromTheme("media-playback-start") : Icons::dynamicIcon;
+        return IS_ACTIVE(entryList.at(index.row()).name) ? QIcon::fromTheme("media-playback-start") : Icons::dynamicRuleIcon;
     case ItemView::Role_SubText: {
         #ifdef ENABLE_KDE_SUPPORT
         return i18np("1 Rule", "%1 Rules", entryList.at(index.row()).rules.count());
@@ -147,8 +152,11 @@ QVariant Dynamic::data(const QModelIndex &index, int role) const
         return QTP_RULES_STR(entryList.at(index.row()).rules.count());
         #endif
     }
-    case ItemView::Role_ToggleIcon:
-        return Icon(IS_ACTIVE(entryList.at(index.row()).name) ? "process-stop" : "media-playback-start");
+    case ItemView::Role_Actions: {
+        QVariant v;
+        v.setValue<QList<Action *> >(QList<Action *>() << (IS_ACTIVE(entryList.at(index.row()).name) ? stopAction : startAction));
+        return v;
+    }
     default:
         return QVariant();
     }
@@ -198,11 +206,19 @@ bool Dynamic::save(const Entry &e)
             return false;
         }
 
+        #if QT_VERSION < 0x050000
         QUrl url(dynamicUrl+"/"+constSaveCmd);
         url.addQueryItem("name", e.name);
         url.addQueryItem(constStatusTime, QString::number(statusTime));
+        #else
+        QUrl url(dynamicUrl+"/"+constSaveCmd);
+        QUrlQuery query;
+        query.addQueryItem("name", e.name);
+        query.addQueryItem(constStatusTime, QString::number(statusTime));
+        url.setQuery(query);
+        #endif
 
-        currentJob=network()->post(QNetworkRequest(url), string.toUtf8());
+        currentJob=NetworkAccessManager::self()->post(QNetworkRequest(url), string.toUtf8());
         connect(currentJob, SIGNAL(finished()), this, SLOT(jobFinished()));
         currentCommand=constSaveCmd;
         currentArgs.clear();
@@ -389,17 +405,7 @@ int Dynamic::getPid() const
 
 bool Dynamic::controlApp(bool isStart)
 {
-    #ifdef ENABLE_KDE_SUPPORT
-    QString cmd=KStandardDirs::findExe("cantata-dynamic"); // , KStandardDirs::installPath("libexec"));
-
-    if (cmd.isEmpty()) {
-        emit error(i18n("Failed to locate the 'cantata-dynamic' helper script."));
-        return false;
-    }
-    #else
-    QString cmd=QLatin1String("cantata-dynamic");
-    #endif
-
+    QString cmd=QLatin1String(INSTALL_PREFIX"/share/cantata/scripts/cantata-dynamic");
     QProcess process;
 
     if (isStart) {
@@ -625,14 +631,6 @@ void Dynamic::checkResponse(const QString &response)
     }
 }
 
-NetworkAccessManager * Dynamic::network()
-{
-    if (!manager) {
-        manager=new NetworkAccessManager(this);
-    }
-    return manager;
-}
-
 void Dynamic::sendCommand(const QString &cmd, const QStringList &args)
 {
     if (currentCommand==constStatusCmd) {
@@ -669,25 +667,41 @@ void Dynamic::sendCommand(const QString &cmd, const QStringList &args)
 
     if (constDeleteCmd==cmd) {
         if (!args.isEmpty()) {
+            #if QT_VERSION < 0x050000
             QUrl url(dynamicUrl+"/"+args.at(0));
             url.addQueryItem(constStatusTime, QString::number(statusTime));
-            currentJob=network()->deleteResource(QNetworkRequest(url));
+            #else
+            QUrl url(dynamicUrl+"/"+args.at(0));
+            QUrlQuery query;
+            query.addQueryItem(constStatusTime, QString::number(statusTime));
+            url.setQuery(query);
+            #endif
+        
+            currentJob=NetworkAccessManager::self()->deleteResource(QNetworkRequest(url));
         } else {
             currentCommand.clear();
             currentArgs.clear();
         }
     } else {
         QUrl url(dynamicUrl+"/"+currentCommand);
+        #if QT_VERSION < 0x050000
+        QUrl &query=url;
+        #else
+        QUrlQuery query;
+        #endif
         if (!args.isEmpty() && 0==args.size()%2) {
             for (int i=0; i<args.size(); i+=2) {
-                url.addQueryItem(args.at(i), args.at(i+1));
+                query.addQueryItem(args.at(i), args.at(i+1));
             }
         }
-        url.addQueryItem(constStatusTime, QString::number(statusTime));
+        query.addQueryItem(constStatusTime, QString::number(statusTime));
+        #if QT_VERSION >= 0x050000
+        url.setQuery(query);
+        #endif
         if (constSetActiveCmd==cmd || constControlCmd==cmd) {
-            currentJob=network()->post(QNetworkRequest(url), QByteArray());
+            currentJob=NetworkAccessManager::self()->post(QNetworkRequest(url), QByteArray());
         } else {
-            currentJob=network()->get(QNetworkRequest(url));
+            currentJob=NetworkAccessManager::self()->get(QNetworkRequest(url));
         }
     }
     if (constListCmd==cmd) {
