@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -26,7 +26,7 @@
 
 #include "ui_playbacksettings.h"
 #include "output.h"
-#include <QtCore/QList>
+#include <QList>
 
 class PlaybackSettings : public QWidget, private Ui::PlaybackSettings
 {
