@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -33,14 +33,15 @@
 #include <KDE/KGlobal>
 #include <KDE/KConfig>
 #include "kwallet.h"
-#include <QtGui/QApplication>
-#include <QtGui/QWidget>
-#include <QtCore/QTimer>
+#include <QApplication>
+#include <QWidget>
+#include <QTimer>
 
 K_GLOBAL_STATIC(Settings, instance)
 #endif
-#include <QtCore/QFile>
-#include <QtCore/QDir>
+#include <QFile>
+#include <QDir>
+#include <qglobal.h>
 
 Settings * Settings::self()
 {
@@ -254,14 +255,27 @@ QString Settings::iconTheme()
 {
     return GET_STRING("iconTheme", QString());
 }
-
 #endif
 
+bool Settings::monoSidebarIcons()
+{
+    #ifdef Q_OS_WIN
+    return GET_BOOL("monoSidebarIcons", false);
+    #else
+    return GET_BOOL("monoSidebarIcons", true);
+    #endif
+}
+
 bool Settings::showPlaylist()
 {
     return GET_BOOL("showPlaylist", true);
 }
 
+bool Settings::showFullScreen()
+{
+    return GET_BOOL("showFullScreen", false);
+}
+
 QByteArray Settings::playQueueHeaderState()
 {
     return GET_BYTE_ARRAY("playQueueHeaderState");
@@ -312,14 +326,9 @@ bool Settings::stopDynamizerOnExit()
     return GET_BOOL("stopDynamizerOnExit", false);
 }
 
-bool Settings::smallPlaybackButtons()
+bool Settings::stopAfterCurrent()
 {
-    return GET_BOOL("smallPlaybackButtons", false);
-}
-
-bool Settings::smallControlButtons()
-{
-    return GET_BOOL("smallControlButtons", false);
+    return GET_BOOL("stopAfterCurrent", false);
 }
 
 bool Settings::storeCoversInMpdDir()
@@ -332,29 +341,56 @@ bool Settings::storeLyricsInMpdDir()
     return GET_BOOL("storeLyricsInMpdDir", true);
 }
 
+bool Settings::storeStreamsInMpdDir()
+{
+    #ifdef Q_OS_WIN
+    bool def=false;
+    #else
+    bool def=version()>=CANTATA_MAKE_VERSION(0, 9, 50);
+    #endif
+    return GET_BOOL("storeStreamsInMpdDir", def);
+}
+
 int Settings::libraryView()
 {
-    return GET_INT("libraryView", (int)(version()>=CANTATA_MAKE_VERSION(0, 5, 0) ? ItemView::Mode_Tree : ItemView::Mode_List));
+    int v=version();
+    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
+                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree)
+                                    : ItemView::Mode_List);
+    return ItemView::toMode(GET_STRING("libraryView", def));
 }
 
 int Settings::albumsView()
 {
-    return GET_INT("albumsView", (int)ItemView::Mode_IconTop);
+    return ItemView::toMode(GET_STRING("albumsView", ItemView::modeStr(ItemView::Mode_IconTop)));
 }
 
 int Settings::folderView()
 {
-    return GET_INT("folderView", (int)ItemView::Mode_Tree);
+    return ItemView::toMode(GET_STRING("folderView", ItemView::modeStr(ItemView::Mode_SimpleTree)));
 }
 
 int Settings::playlistsView()
 {
-    return GET_INT("playlistsView", (int)(version()>=CANTATA_MAKE_VERSION(0, 5, 0) ? ItemView::Mode_Tree : ItemView::Mode_List));
+    int v=version();
+    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
+                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_GroupedTree)
+                                    : ItemView::Mode_List);
+    return ItemView::toMode(GET_STRING("playlistsView", def));
 }
 
 int Settings::streamsView()
 {
-    return GET_INT("streamsView", (int)(version()>=CANTATA_MAKE_VERSION(0, 5, 0) ? ItemView::Mode_Tree : ItemView::Mode_List));
+    int v=version();
+    QString def=ItemView::modeStr(v>=CANTATA_MAKE_VERSION(0, 5, 0)
+                                    ? (v<CANTATA_MAKE_VERSION(0, 9, 50) ? ItemView::Mode_SimpleTree : ItemView::Mode_DetailedTree)
+                                    : ItemView::Mode_List);
+    return ItemView::toMode(GET_STRING("streamsView", def));
+}
+
+int Settings::onlineView()
+{
+    return ItemView::toMode(GET_STRING("onlineView", ItemView::modeStr(ItemView::Mode_DetailedTree)));
 }
 
 bool Settings::libraryArtistImage()
@@ -364,12 +400,14 @@ bool Settings::libraryArtistImage()
 
 int Settings::libraryCoverSize()
 {
-    return GET_INT("libraryCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
+    int size=GET_INT("libraryCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
+    return size>MusicLibraryItemAlbum::CoverExtraLarge || size<0 ? MusicLibraryItemAlbum::CoverMedium : size;
 }
 
 int Settings::albumsCoverSize()
 {
-    return GET_INT("albumsCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
+    int size=GET_INT("albumsCoverSize", (int)(MusicLibraryItemAlbum::CoverMedium));
+    return size>MusicLibraryItemAlbum::CoverExtraLarge || size<0 ? MusicLibraryItemAlbum::CoverMedium : size;
 }
 
 int Settings::albumSort()
@@ -435,7 +473,13 @@ bool Settings::lyricsBgnd()
 
 int Settings::infoZoom()
 {
-    return GET_INT("infoZoom", 0);
+    int zoom=GET_INT("infoZoom", 0);
+    if (zoom>0 && version()<CANTATA_MAKE_VERSION(0, 9, 51)) {
+        modified=true;
+        zoom=0;
+        SET_VALUE("infoZoom", zoom);
+    }
+    return zoom;
 }
 
 QString Settings::page()
@@ -446,8 +490,7 @@ QString Settings::page()
 QStringList Settings::hiddenPages()
 {
     QStringList def;
-    def << "FolderPage"
-        << "ServerInfoPage";
+    def << "FolderPage";
     return GET_STRINGLIST("hiddenPages", def);
 }
 
@@ -469,7 +512,7 @@ bool Settings::showDeleteAction()
 
 int Settings::devicesView()
 {
-    return GET_INT("devicesView", (int)ItemView::Mode_Tree);
+    return GET_INT("devicesView", (int)ItemView::Mode_DetailedTree);
 }
 #endif
 
@@ -481,7 +524,7 @@ int Settings::version()
         if (3==parts.size()) {
             ver=CANTATA_MAKE_VERSION(parts.at(0).toInt(), parts.at(1).toInt(), parts.at(2).toInt());
         } else {
-            ver=CANTATA_MAKE_VERSION(0, 8, 0);
+            ver=CANTATA_MAKE_VERSION(0, 9, 0);
             SET_VALUE("version", PACKAGE_VERSION);
         }
     }
@@ -502,7 +545,12 @@ int Settings::stopFadeDuration()
 #ifdef TAGLIB_FOUND
 int Settings::httpPort()
 {
-    return GET_INT("httpPort", 9001);
+    return GET_INT("httpPort", 0);
+}
+
+int Settings::httpAllocatedPort()
+{
+    return GET_INT("httpAllocatedPort", 0);
 }
 
 QString Settings::httpAddress()
@@ -512,12 +560,14 @@ QString Settings::httpAddress()
 
 bool Settings::enableHttp()
 {
-    return GET_BOOL("enableHttp", false);
+    bool def=version()>CANTATA_MAKE_VERSION(0, 9, 50);
+    return GET_BOOL("enableHttp", def);
 }
 
 bool Settings::alwaysUseHttp()
 {
-    return GET_BOOL("alwaysUseHttp", false);
+    bool def=version()>CANTATA_MAKE_VERSION(0, 9, 50);
+    return GET_BOOL("alwaysUseHttp", def);
 }
 #endif
 
@@ -558,6 +608,48 @@ QString Settings::streamUrl()
 }
 #endif
 
+#if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+bool Settings::cdAuto()
+{
+    return GET_BOOL("cdAuto", true);
+}
+#ifdef LAME_FOUND
+bool Settings::cdEncode()
+{
+    return GET_BOOL("cdEncode", true);
+}
+#endif
+#endif
+
+#if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
+bool Settings::useCddb()
+{
+    return GET_BOOL("useCddb", true);
+}
+#endif
+
+#ifdef CDDB_FOUND
+QString Settings::cddbHost()
+{
+    return GET_STRING("cddbHost", QString("freedb.freedb.org"));
+}
+
+int Settings::cddbPort()
+{
+    return GET_INT("cddbPort", 8880);
+}
+
+bool Settings::paranoiaFull()
+{
+    return GET_BOOL("paranoiaFull", true);
+}
+
+bool Settings::paranoiaNeverSkip()
+{
+    return GET_BOOL("paranoiaNeverSkip", true);
+}
+#endif
+
 bool Settings::forceSingleClick()
 {
     return GET_BOOL("forceSingleClick", true);
@@ -568,6 +660,16 @@ bool Settings::startHidden()
     return GET_BOOL("startHidden", false);
 }
 
+QString Settings::amazonAccessKey()
+{
+    return GET_STRING("amazonAccessKey", QString());
+}
+
+QString Settings::amazonSecretAccessKey()
+{
+    return GET_STRING("amazonSecretAccessKey", QString());
+}
+
 void Settings::removeConnectionDetails(const QString &v)
 {
     if (v==currentConnection()) {
@@ -624,280 +726,213 @@ void Settings::saveConnectionDetails(const MPDConnectionDetails &v)
     modified=true;
 }
 
-#define SET_VALUE_MOD(KEY, VAL) modified=true; SET_VALUE(KEY, VAL)
+#define SET_VALUE_MOD(KEY) if (v!=KEY()) { modified=true; SET_VALUE(#KEY, v); }
+#define SET_ITEMVIEW_MODE_VALUE_MOD(KEY) if (v!=KEY()) { modified=true; SET_VALUE(#KEY, ItemView::modeStr((ItemView::Mode)v)); }
 
 void Settings::saveCurrentConnection(const QString &v)
 {
-    if (v!=currentConnection()) {
-        SET_VALUE_MOD("currentConnection", v);
-    }
+    SET_VALUE_MOD(currentConnection)
 }
 
 void Settings::saveShowPlaylist(bool v)
 {
-    if (v!=showPlaylist()) {
-        SET_VALUE_MOD("showPlaylist", v);
-    }
+    SET_VALUE_MOD(showPlaylist)
+}
+
+void Settings::saveShowFullScreen(bool v)
+{
+    SET_VALUE_MOD(showFullScreen)
 }
 
 void Settings::savePlayQueueHeaderState(const QByteArray &v)
 {
-    if (v!=playQueueHeaderState()) {
-        SET_VALUE_MOD("playQueueHeaderState", v);
-    }
+    SET_VALUE_MOD(playQueueHeaderState)
 }
 
 void Settings::saveSplitterState(const QByteArray &v)
 {
-    if (v!=splitterState()) {
-        SET_VALUE_MOD("splitterState", v);
-    }
+    SET_VALUE_MOD(splitterState)
 }
 
 void Settings::saveSplitterAutoHide(bool v)
 {
-    if (v!=splitterAutoHide()) {
-        SET_VALUE_MOD("splitterAutoHide", v);
-    }
+    SET_VALUE_MOD(splitterAutoHide)
 }
 
 void Settings::saveMainWindowSize(const QSize &v)
 {
-    if (v!=mainWindowSize()) {
-        SET_VALUE_MOD("mainWindowSize", v);
-    }
+    SET_VALUE_MOD(mainWindowSize)
 }
 
 void Settings::saveMainWindowCollapsedSize(const QSize &v)
 {
-    if (v!=mainWindowCollapsedSize()) {
-        SET_VALUE_MOD("mainWindowCollapsedSize", v);
-    }
+    SET_VALUE_MOD(mainWindowCollapsedSize)
 }
 
 void Settings::saveUseSystemTray(bool v)
 {
-    if (v!=useSystemTray()) {
-        SET_VALUE_MOD("useSystemTray", v);
-    }
+    SET_VALUE_MOD(useSystemTray)
 }
 
 void Settings::saveMinimiseOnClose(bool v)
 {
-    if (v!=minimiseOnClose()) {
-        SET_VALUE_MOD("minimiseOnClose", v);
-    }
+    SET_VALUE_MOD(minimiseOnClose)
 }
 
 void Settings::saveShowPopups(bool v)
 {
-    if (v!=showPopups()) {
-        SET_VALUE_MOD("showPopups", v);
-    }
+    SET_VALUE_MOD(showPopups)
 }
 
 void Settings::saveStopOnExit(bool v)
 {
-    if (v!=stopOnExit()) {
-        SET_VALUE_MOD("stopOnExit", v);
-    }
+    SET_VALUE_MOD(stopOnExit)
 }
 
 void Settings::saveStopDynamizerOnExit(bool v)
 {
-    if (v!=stopDynamizerOnExit()) {
-        SET_VALUE_MOD("stopDynamizerOnExit", v);
-    }
+    SET_VALUE_MOD(stopDynamizerOnExit)
 }
 
-void Settings::saveSmallPlaybackButtons(bool v)
+void Settings::saveStopAfterCurrent(bool v)
 {
-    if (v!=smallPlaybackButtons()) {
-        SET_VALUE_MOD("smallPlaybackButtons", v);
-    }
-}
-
-void Settings::saveSmallControlButtons(bool v)
-{
-    if (v!=smallControlButtons()) {
-        SET_VALUE_MOD("smallControlButtons", v);
-    }
+    SET_VALUE_MOD(stopAfterCurrent)
 }
 
 void Settings::saveStoreCoversInMpdDir(bool v)
 {
-    if (v!=storeCoversInMpdDir()) {
-        SET_VALUE_MOD("storeCoversInMpdDir", v);
-    }
+    SET_VALUE_MOD(storeCoversInMpdDir)
 }
 
 void Settings::saveStoreLyricsInMpdDir(bool v)
 {
-    if (v!=storeLyricsInMpdDir()) {
-        SET_VALUE_MOD("storeLyricsInMpdDir", v);
-    }
+    SET_VALUE_MOD(storeLyricsInMpdDir)
+}
+
+void Settings::saveStoreStreamsInMpdDir(bool v)
+{
+    SET_VALUE_MOD(storeStreamsInMpdDir)
 }
 
 void Settings::saveLibraryView(int v)
 {
-    if (v!=libraryView()) {
-        SET_VALUE_MOD("libraryView", v);
-    }
+    SET_ITEMVIEW_MODE_VALUE_MOD(libraryView)
 }
 
 void Settings::saveAlbumsView(int v)
 {
-    if (v!=albumsView()) {
-        SET_VALUE_MOD("albumsView", v);
-    }
+    SET_ITEMVIEW_MODE_VALUE_MOD(albumsView)
 }
 
 void Settings::saveFolderView(int v)
 {
-    if (v!=folderView()) {
-        SET_VALUE_MOD("folderView", v);
-    }
+    SET_ITEMVIEW_MODE_VALUE_MOD(folderView)
 }
 
 void Settings::savePlaylistsView(int v)
 {
-    if (v!=playlistsView()) {
-        SET_VALUE_MOD("playlistsView", v);
-    }
+    SET_ITEMVIEW_MODE_VALUE_MOD(playlistsView)
 }
 
 void Settings::saveStreamsView(int v)
 {
-    if (v!=streamsView()) {
-        SET_VALUE_MOD("streamsView", v);
-    }
+    SET_ITEMVIEW_MODE_VALUE_MOD(streamsView)
+}
+
+void Settings::saveOnlineView(int v)
+{
+    SET_ITEMVIEW_MODE_VALUE_MOD(onlineView)
 }
 
 void Settings::saveLibraryArtistImage(bool v)
 {
-    if (v!=libraryArtistImage()) {
-        SET_VALUE_MOD("libraryArtistImage", v);
-    }
+    SET_VALUE_MOD(libraryArtistImage)
 }
 
 void Settings::saveLibraryCoverSize(int v)
 {
-    if (v!=libraryCoverSize()) {
-        SET_VALUE_MOD("libraryCoverSize", v);
-    }
+    SET_VALUE_MOD(libraryCoverSize)
 }
 
 void Settings::saveAlbumsCoverSize(int v)
 {
-    if (v!=albumsCoverSize()) {
-        SET_VALUE_MOD("albumsCoverSize", v);
-    }
+    SET_VALUE_MOD(albumsCoverSize)
 }
 
 void Settings::saveAlbumSort(int v)
 {
-    if (v!=albumSort()) {
-        SET_VALUE_MOD("albumSort", v);
-    }
+    SET_VALUE_MOD(albumSort)
 }
 
 void Settings::saveSidebar(int v)
 {
-    if (v!=sidebar()) {
-        SET_VALUE_MOD("sidebar", v);
-    }
+    SET_VALUE_MOD(sidebar)
 }
 
 void Settings::saveLibraryYear(bool v)
 {
-    if (v!=libraryYear()) {
-        SET_VALUE_MOD("libraryYear", v);
-    }
+    SET_VALUE_MOD(libraryYear)
 }
 
 void Settings::saveGroupSingle(bool v)
 {
-    if (v!=groupSingle()) {
-        SET_VALUE_MOD("groupSingle", v);
-    }
+    SET_VALUE_MOD(groupSingle)
 }
 
 void Settings::saveGroupMultiple(bool v)
 {
-    if (v!=groupMultiple()) {
-        SET_VALUE_MOD("groupMultiple", v);
-    }
+    SET_VALUE_MOD(groupMultiple)
 }
 
-void Settings::saveLyricProviders(const QStringList &p)
+void Settings::saveLyricProviders(const QStringList &v)
 {
-    if (p!=lyricProviders()) {
-        SET_VALUE_MOD("lyricProviders", p);
-    }
+    SET_VALUE_MOD(lyricProviders)
 }
 
 void Settings::saveLyricsZoom(int v)
 {
-    if (v!=lyricsZoom()) {
-        SET_VALUE_MOD("lyricsZoom", v);
-    }
+    SET_VALUE_MOD(lyricsZoom)
 }
 
 void Settings::saveLyricsBgnd(bool v)
 {
-    if (v!=lyricsBgnd()) {
-        SET_VALUE_MOD("lyricsBgnd", v);
-    }
+    SET_VALUE_MOD(lyricsBgnd)
 }
 
 void Settings::saveInfoZoom(int v)
 {
-    if (v!=infoZoom()) {
-        SET_VALUE_MOD("infoZoom", v);
-    }
+    SET_VALUE_MOD(infoZoom)
 }
 
 void Settings::savePage(const QString &v)
 {
-    if (v!=page()) {
-        SET_VALUE_MOD("page", v);
-    }
+    SET_VALUE_MOD(page)
 }
 
-void Settings::saveHiddenPages(const QStringList &p)
+void Settings::saveHiddenPages(const QStringList &v)
 {
-    if (p!=hiddenPages()) {
-        SET_VALUE_MOD("hiddenPages", p);
-    }
+    SET_VALUE_MOD(hiddenPages)
 }
 
 void Settings::saveGnomeMediaKeys(bool v)
 {
-    if (v!=gnomeMediaKeys()) {
-        SET_VALUE_MOD("gnomeMediaKeys", v);
-    }
+    SET_VALUE_MOD(gnomeMediaKeys)
 }
 
 #ifdef ENABLE_DEVICES_SUPPORT
 void Settings::saveOverwriteSongs(bool v)
 {
-    if (v!=overwriteSongs()) {
-        SET_VALUE_MOD("overwriteSongs", v);
-    }
+    SET_VALUE_MOD(overwriteSongs)
 }
 
 void Settings::saveShowDeleteAction(bool v)
 {
-    if (v!=showDeleteAction()) {
-        SET_VALUE_MOD("showDeleteAction", v);
-    }
+    SET_VALUE_MOD(showDeleteAction)
 }
 
 void Settings::saveDevicesView(int v)
 {
-    if (v!=devicesView()) {
-        SET_VALUE_MOD("devicesView", v);
-    }
+    SET_VALUE_MOD(devicesView)
 }
 #endif
 
@@ -908,111 +943,131 @@ void Settings::saveStopFadeDuration(int v)
     } else if (v>MaxFade) {
         v=MaxFade;
     }
-    if (v!=stopFadeDuration()) {
-        SET_VALUE_MOD("stopFadeDuration", v);
-    }
+    SET_VALUE_MOD(stopFadeDuration)
 }
 
 #ifdef TAGLIB_FOUND
 void Settings::saveHttpPort(int v)
 {
-    if (v!=httpPort()) {
-        SET_VALUE_MOD("httpPort", v);
-    }
+    SET_VALUE_MOD(httpPort)
+}
+
+void Settings::saveHttpAllocatedPort(int v)
+{
+    SET_VALUE_MOD(httpAllocatedPort)
 }
 
 void Settings::saveHttpAddress(const QString &v)
 {
-    if (v!=httpAddress()) {
-        SET_VALUE_MOD("httpAddress", v);
-    }
+    SET_VALUE_MOD(httpAddress)
 }
 
 void Settings::saveEnableHttp(bool v)
 {
-    if (v!=enableHttp()) {
-        SET_VALUE_MOD("enableHttp", v);
-    }
+    SET_VALUE_MOD(enableHttp)
 }
 
 void Settings::saveAlwaysUseHttp(bool v)
 {
-    if (v!=alwaysUseHttp()) {
-        SET_VALUE_MOD("alwaysUseHttp", v);
-    }
+    SET_VALUE_MOD(alwaysUseHttp)
 }
 #endif
 
 void Settings::savePlayQueueGrouped(bool v)
 {
-    if (v!=playQueueGrouped()) {
-        SET_VALUE_MOD("playQueueGrouped", v);
-    }
+    SET_VALUE_MOD(playQueueGrouped)
 }
 
 void Settings::savePlayQueueAutoExpand(bool v)
 {
-    if (v!=playQueueAutoExpand()) {
-        SET_VALUE_MOD("playQueueAutoExpand", v);
-    }
+    SET_VALUE_MOD(playQueueAutoExpand)
 }
 
 void Settings::savePlayQueueStartClosed(bool v)
 {
-    if (v!=playQueueStartClosed()) {
-        SET_VALUE_MOD("playQueueStartClosed", v);
-    }
+    SET_VALUE_MOD(playQueueStartClosed)
 }
 
 void Settings::savePlayQueueScroll(bool v)
 {
-    if (v!=playQueueScroll()) {
-        SET_VALUE_MOD("playQueueScroll", v);
-    }
+    SET_VALUE_MOD(playQueueScroll)
 }
 
 void Settings::savePlayListsStartClosed(bool v)
 {
-    if (v!=playListsStartClosed()) {
-        SET_VALUE_MOD("playListsStartClosed", v);
-    }
+    SET_VALUE_MOD(playListsStartClosed)
 }
 
 #ifdef PHONON_FOUND
 void Settings::savePlayStream(bool v)
 {
-    if (v!=playStream()) {
-        SET_VALUE_MOD("playStream", v);
-    }
+    SET_VALUE_MOD(playStream)
 }
 
 void Settings::saveStreamUrl(const QString &v)
 {
-    if (v!=streamUrl()) {
-        SET_VALUE_MOD("streamUrl", v);
-    }
+    SET_VALUE_MOD(streamUrl)
+}
+#endif
+
+#if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+void Settings::saveCdAuto(bool v)
+{
+    SET_VALUE_MOD(cdAuto)
+}
+#ifdef LAME_FOUND
+void Settings::saveCdEncode(bool v)
+{
+    SET_VALUE_MOD(cdEncode)
+}
+#endif
+#endif
+
+#if defined CDDB_FOUND && defined MUSICBRAINZ5_FOUND
+void Settings::saveUseCddb(bool v)
+{
+    SET_VALUE_MOD(useCddb)
+}
+#endif
+
+#ifdef CDDB_FOUND
+void Settings::saveCddbHost(const QString &v)
+{
+    SET_VALUE_MOD(cddbHost)
+}
+
+void Settings::saveCddbPort(int v)
+{
+    SET_VALUE_MOD(cddbPort)
+}
+
+void Settings::saveParanoiaFull(bool v)
+{
+    SET_VALUE_MOD(paranoiaFull)
+}
+
+void Settings::saveParanoiaNeverSkip(bool v)
+{
+    SET_VALUE_MOD(paranoiaNeverSkip)
 }
 #endif
 
 void Settings::saveForceSingleClick(bool v)
 {
-    if (v!=forceSingleClick()) {
-        SET_VALUE_MOD("forceSingleClick", v);
-    }
+    SET_VALUE_MOD(forceSingleClick)
 }
 
 void Settings::saveStartHidden(bool v)
 {
-    if (v!=startHidden()) {
-        SET_VALUE_MOD("startHidden", v);
-    }
+    SET_VALUE_MOD(startHidden);
 }
 
 void Settings::save(bool force)
 {
     if (force) {
-        if (version()!=PACKAGE_VERSION) {
-            SET_VALUE_MOD("version", PACKAGE_VERSION_STRING);
+        if (version()!=PACKAGE_VERSION || isFirstRun) {
+            modified=true;
+            SET_VALUE("version", PACKAGE_VERSION_STRING);
             ver=PACKAGE_VERSION;
         }
         if (modified) {
