@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /* This file is part of Clementine.
@@ -24,37 +24,30 @@
 #ifndef LYRICSETTINGS_H
 #define LYRICSETTINGS_H
 
-#include <QtGui/QWidget>
-#include <QtCore/QStringList>
-#include <QtCore/QList>
+#include <QWidget>
+#include <QStringList>
+#include <QList>
+#include "ui_lyricsettings.h"
 
-class Ui_LyricSettings;
 class QListWidgetItem;
 class UltimateLyricsProvider;
 
-class LyricSettings : public QWidget
+class LyricSettings : public QWidget, private Ui::LyricSettings
 {
   Q_OBJECT
 
 public:
-  LyricSettings(QWidget *parent = 0);
-  ~LyricSettings();
+    LyricSettings(QWidget *p);
+    virtual ~LyricSettings() { }
 
-  QStringList EnabledProviders();
-
-public Q_SLOTS:
-  void Load(const QList<UltimateLyricsProvider*> &providers);
+    void load();
+    void save();
 
 private Q_SLOTS:
-  void MoveUp();
-  void MoveDown();
-  void Move(int d);
-
-  void CurrentItemChanged(QListWidgetItem* item);
-//   void ItemChanged(QListWidgetItem* item);
-
-private:
-  Ui_LyricSettings* ui_;
+    void moveUp();
+    void moveDown();
+    void move(int d);
+    void currentItemChanged(QListWidgetItem *item);
 };
 
 #endif // LYRICSETTINGS_H
