@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -36,9 +36,14 @@
 #include "localize.h"
 #include "icons.h"
 #include "mountpoints.h"
-#include <QtGui/QMenu>
-#include <QtCore/QStringList>
-#include <QtCore/QMimeData>
+#include "stdactions.h"
+#include "actioncollection.h"
+#if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+#include "audiocddevice.h"
+#endif
+#include <QMenu>
+#include <QStringList>
+#include <QMimeData>
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KIcon>
 #include <KDE/KGlobal>
@@ -49,6 +54,7 @@
 #include <solid/storageaccess.h>
 #include <solid/storagedrive.h>
 #include <solid/storagevolume.h>
+#include <solid/opticaldisc.h>
 K_GLOBAL_STATIC(DevicesModel, instance)
 #else
 #include "solid-lite/device.h"
@@ -58,11 +64,26 @@ K_GLOBAL_STATIC(DevicesModel, instance)
 #include "solid-lite/storageaccess.h"
 #include "solid-lite/storagedrive.h"
 #include "solid-lite/storagevolume.h"
+#include "solid-lite/opticaldisc.h"
 #endif
 
-#include <QtCore/QDebug>
+#include <QDebug>
 #define DBUG qDebug()
 
+QString DevicesModel::fixDevicePath(const QString &path)
+{
+    // Remove MTP IDs, and display storage...
+    if (path.startsWith(QChar('{')) && path.contains(QChar('}'))) {
+        int end=path.indexOf(QChar('}'));
+        QStringList details=path.mid(1, end-1).split(QChar('/'));
+        if (details.length()>3) {
+            return QChar('(')+details.at(3)+QLatin1String(") ")+path.mid(end+1);
+        }
+        return path.mid(end+1);
+    }
+    return path;
+}
+
 DevicesModel * DevicesModel::self()
 {
     #ifdef ENABLE_KDE_SUPPORT
@@ -77,11 +98,18 @@ DevicesModel * DevicesModel::self()
 }
 
 DevicesModel::DevicesModel(QObject *parent)
-    : QAbstractItemModel(parent)
+    : ActionModel(parent)
     , itemMenu(0)
     , enabled(false)
     , inhibitMenuUpdate(false)
 {
+    configureAction = ActionCollection::get()->createAction("configuredevice", i18n("Configure Device"), Icons::configureIcon);
+    refreshAction = ActionCollection::get()->createAction("refreshdevice", i18n("Refresh Device"), "view-refresh");
+    connectAction = ActionCollection::get()->createAction("connectdevice", i18n("Connect Device"), Icons::connectIcon);
+    disconnectAction = ActionCollection::get()->createAction("disconnectdevice", i18n("Disconnect Device"), Icons::disconnectIcon);
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    editAction = ActionCollection::get()->createAction("editcd", i18n("Edit CD Details"), Icons::editIcon);
+    #endif
     updateItemMenu();
 }
 
@@ -157,6 +185,12 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
     case Qt::DecorationRole:
         switch (item->itemType()) {
         case MusicLibraryItem::Type_Root: {
+            #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+            if (Device::AudioCd==static_cast<Device *>(item)->devType() &&
+                !static_cast<AudioCdDevice *>(item)->cover().img.isNull()) {
+                return static_cast<AudioCdDevice *>(item)->cover().img;
+            }
+            #endif
             QString iconName = static_cast<MusicLibraryItemRoot *>(item)->icon();
             return QIcon::fromTheme(iconName.isEmpty() ? QLatin1String("multimedia-player") : iconName);
         }
@@ -176,11 +210,15 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
     case Qt::DisplayRole:
         if (MusicLibraryItem::Type_Song==item->itemType()) {
             MusicLibraryItemSong *song = static_cast<MusicLibraryItemSong *>(item);
-            if (static_cast<MusicLibraryItemAlbum *>(song->parentItem())->isSingleTracks()) {
-                return song->song().artistSong();
+            if (MusicLibraryItem::Type_Artist==song->parentItem()->itemType()) {
+                if (static_cast<MusicLibraryItemAlbum *>(song->parentItem())->isSingleTracks()) {
+                    return song->song().artistSong();
+                } else {
+                    return song->song().trackAndTitleStr(static_cast<MusicLibraryItemArtist *>(song->parentItem()->parentItem())->isVarious() &&
+                                                         !Song::isVariousArtists(song->song().artist));
+                }
             } else {
-                return song->song().trackAndTitleStr(static_cast<MusicLibraryItemArtist *>(song->parentItem()->parentItem())->isVarious() &&
-                                                     !Song::isVariousArtists(song->song().artist));
+                return song->song().trackAndTitleStr(Song::isVariousArtists(song->song().artist));
             }
         } else if(MusicLibraryItem::Type_Album==item->itemType() && MusicLibraryItemAlbum::showDate() &&
                   static_cast<MusicLibraryItemAlbum *>(item)->year()>0) {
@@ -190,6 +228,18 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
     case Qt::ToolTipRole:
         switch (item->itemType()) {
         case MusicLibraryItem::Type_Root:
+            #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+            if (Device::AudioCd==static_cast<Device *>(item)->devType()) {
+                return 0==item->childCount()
+                    ? item->data()
+                    : item->data()+"\n"+
+                        #ifdef ENABLE_KDE_SUPPORT
+                        i18np("1 Track (%2)", "%1 Tracks (%2)",item->childCount(), Song::formattedTime(static_cast<MusicLibraryItemAlbum *>(item)->totalTime()));
+                        #else
+                        QTP_TRACKS_DURATION_STR(item->childCount(), Song::formattedTime(static_cast<AudioCdDevice *>(item)->totalTime()));
+                        #endif
+            }
+            #endif
             return 0==item->childCount()
                 ? item->data()
                 : item->data()+"\n"+
@@ -218,7 +268,9 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
                     #endif
         case MusicLibraryItem::Type_Song:
             return data(index, Qt::DisplayRole).toString()+QLatin1String("<br/>")+Song::formattedTime(static_cast<MusicLibraryItemSong *>(item)->time())+
-                   QLatin1String("<br/><small><i>")+static_cast<MusicLibraryItemSong *>(item)->song().file+QLatin1String("</i></small>");
+                    (static_cast<MusicLibraryItemSong *>(item)->song().file.isEmpty()
+                     ? QString()
+                     : QLatin1String("<br/><small><i>")+fixDevicePath(static_cast<MusicLibraryItemSong *>(item)->song().file)+QLatin1String("</i></small>"));
         default: return QVariant();
         }
     case ItemView::Role_ImageSize:
@@ -235,7 +287,11 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
                 return dev->statusMessage();
             }
             if (!dev->isConnected()) {
-                return i18n("Not Connected");
+                QString sub=dev->subText();
+                return i18n("Not Connected")+(sub.isEmpty() ? QString() : (QString(" - ")+sub));
+            }
+            if (Device::AudioCd==dev->devType()) {
+                return dev->subText();
             }
             #ifdef ENABLE_KDE_SUPPORT
             return i18np("1 Artist", "%1 Artists", item->childCount());
@@ -278,18 +334,45 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
             return static_cast<Device *>(item)->capacityString();
         }
         return QVariant();
-    case ItemView::Role_ToggleIcon:
-        if (MusicLibraryItem::Type_Root==item->itemType() && static_cast<Device *>(item)->supportsDisconnect()) {
-            return static_cast<Device *>(item)->isConnected() ? Icons::disconnectIcon : Icons::connectIcon;
-        }
-        return QVariant();
+    case ItemView::Role_Actions:
+        if (MusicLibraryItem::Type_Root==item->itemType()) {
+            QVariant v;
+            QList<Action *> actions;
+            if (Device::AudioCd!=static_cast<Device *>(item)->devType()) {
+                actions << configureAction;
+            } else if (HttpServer::self()->isAlive()) {
+                actions << StdActions::self()->replacePlayQueueAction;
+            }
+            actions << refreshAction;
+            if (static_cast<Device *>(item)->supportsDisconnect()) {
+                actions << (static_cast<Device *>(item)->isConnected() ? disconnectAction : connectAction);
+            }
+            #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+            if (Device::AudioCd==static_cast<Device *>(item)->devType()) {
+                actions << editAction;
+            }
+            #endif
+            v.setValue<QList<Action *> >(actions);
+            return v;
+        } /*else if (HttpServer::self()->isAlive()) {
+            MusicLibraryItem *root=static_cast<MusicLibraryItem *>(item);
+            while (root && MusicLibraryItem::Type_Root!=root->itemType()) {
+                root=root->parentItem();
+            }
+            if (root && static_cast<Device *>(root)->canPlaySongs()) {
+                QVariant v;
+                v.setValue<QList<Action *> >(QList<Action *>() << StdActions::self()->replacePlayQueueAction << StdActions::self()->addToPlayQueueAction);
+                return v;
+            }
+        } */
+        break;
     default:
         return QVariant();
     }
     return QVariant();
 }
 
-void DevicesModel::clear()
+void DevicesModel::clear(bool clearConfig)
 {
     inhibitMenuUpdate=true;
     QSet<QString> remoteUdis;
@@ -306,7 +389,7 @@ void DevicesModel::clear()
         deviceRemoved(u);
     }
     foreach (const QString &u, remoteUdis) {
-        removeRemoteDevice(u);
+        removeRemoteDevice(u, clearConfig);
     }
 
     devices.clear();
@@ -317,6 +400,8 @@ void DevicesModel::clear()
 
 void DevicesModel::setEnabled(bool e)
 {
+    StdActions::self()->copyToDeviceAction->setVisible(e);
+
     if (e==enabled) {
         return;
     }
@@ -335,17 +420,26 @@ void DevicesModel::setEnabled(bool e)
         loadRemote();
         #endif
     } else {
-        disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString &)), this, SLOT(deviceAdded(const QString &)));
-        disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString &)), this, SLOT(deviceRemoved(const QString &)));
-//        disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
-//                   this, SLOT(setCover(const Song &, const QImage &, const QString &)));
-        disconnect(MountPoints::self(), SIGNAL(updated()), this, SLOT(mountsChanged()));
-        clear();
+        stop();
+        clear(false);
     }
     inhibitMenuUpdate=false;
     updateItemMenu();
 }
 
+void DevicesModel::stop()
+{
+    foreach (Device *dev, devices) {
+        dev->stop();
+    }
+
+    disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString &)), this, SLOT(deviceAdded(const QString &)));
+    disconnect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString &)), this, SLOT(deviceRemoved(const QString &)));
+//        disconnect(Covers::self(), SIGNAL(cover(const Song &, const QImage &, const QString &)),
+//                   this, SLOT(setCover(const Song &, const QImage &, const QString &)));
+    disconnect(MountPoints::self(), SIGNAL(updated()), this, SLOT(mountsChanged()));
+}
+
 Device * DevicesModel::device(const QString &udi)
 {
     int idx=indexOf(udi);
@@ -370,14 +464,12 @@ void DevicesModel::setCover(const Song &song, const QImage &img)
     if (i<0) {
         return;
     }
-    MusicLibraryItemArtist *artistItem = static_cast<MusicLibraryItemRoot *>(dev)->artist(song, false);
+    MusicLibraryItemArtist *artistItem = dev->artist(song, false);
     if (artistItem) {
         MusicLibraryItemAlbum *albumItem = artistItem->album(song, false);
-        if (albumItem) {
-            if (static_cast<const MusicLibraryItemAlbum *>(albumItem)->setCover(img)) {
-                QModelIndex idx=index(artistItem->childItems().indexOf(albumItem), 0, index(static_cast<MusicLibraryItemContainer *>(dev)->childItems().indexOf(artistItem), 0, index(i, 0, QModelIndex())));
-                emit dataChanged(idx, idx);
-            }
+        if (albumItem && albumItem->setCover(img)) {
+            QModelIndex idx=index(artistItem->childItems().indexOf(albumItem), 0, index(dev->childItems().indexOf(artistItem), 0, index(i, 0, QModelIndex())));
+            emit dataChanged(idx, idx);
         }
     }
 }
@@ -397,15 +489,24 @@ void DevicesModel::deviceUpdating(const QString &udi, bool state)
             }
             QModelIndex modelIndex=createIndex(idx, 0, dev);
             emit dataChanged(modelIndex, modelIndex);
-            QSet<QString> genres;
-            foreach (Device *dev, devices) {
-                genres+=dev->genres();
-            }
-            emit updateGenres(genres);
+            updateGenres();
+            emit updated(modelIndex);
         }
     }
 }
 
+void DevicesModel::updateGenres()
+{
+    QSet<QString> newGenres;
+    foreach (Device *dev, devices) {
+        newGenres+=dev->genres();
+    }
+    if (newGenres!=devGenres) {
+        devGenres=newGenres;
+        emit updateGenres(devGenres);
+    }
+}
+
 Qt::ItemFlags DevicesModel::flags(const QModelIndex &index) const
 {
     if (index.isValid()) {
@@ -424,6 +525,16 @@ QStringList DevicesModel::filenames(const QModelIndexList &indexes, bool playabl
     return fnames;
 }
 
+QStringList DevicesModel::playableUrls(const QModelIndexList &indexes) const
+{
+    QList<Song> songList=songs(indexes, true, true);
+    QStringList urls;
+    foreach (const Song &s, songList) {
+        urls.append(HttpServer::self()->encodeUrl(s));
+    }
+    return urls;
+}
+
 static Song fixPath(Song s, const QString &path)
 {
     if (!path.isEmpty()) {
@@ -455,19 +566,27 @@ QList<Song> DevicesModel::songs(const QModelIndexList &indexes, bool playableOnl
 
         switch (item->itemType()) {
         case MusicLibraryItem::Type_Root: {
-            // First, sort all artists as they would appear in UI...
-            QList<MusicLibraryItem *> artists=static_cast<const MusicLibraryItemContainer *>(item)->childItems();
-            qSort(artists.begin(), artists.end(), MusicLibraryItemArtist::lessThan);
-            foreach (MusicLibraryItem *a, artists) {
-                const MusicLibraryItemContainer *artist=static_cast<const MusicLibraryItemContainer *>(a);
-                // Now sort all albums as they would appear in UI...
-                QList<MusicLibraryItem *> artistAlbums=static_cast<const MusicLibraryItemContainer *>(artist)->childItems();
-                qSort(artistAlbums.begin(), artistAlbums.end(), MusicLibraryItemAlbum::lessThan);
-                foreach (MusicLibraryItem *i, artistAlbums) {
-                    const MusicLibraryItemContainer *album=static_cast<const MusicLibraryItemContainer *>(i);
-                    foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
-                        if (MusicLibraryItem::Type_Song==song->itemType() && !devSongs[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->song())) {
-                            devSongs[parent] << fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), base);
+            if (Device::AudioCd==static_cast<Device *>(parent)->devType()) {
+                foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(item)->childItems()) {
+                    if (MusicLibraryItem::Type_Song==song->itemType() && !devSongs[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->song())) {
+                        devSongs[parent] << fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), base);
+                    }
+                }
+            } else {
+                // First, sort all artists as they would appear in UI...
+                QList<MusicLibraryItem *> artists=static_cast<const MusicLibraryItemContainer *>(item)->childItems();
+                qSort(artists.begin(), artists.end(), MusicLibraryItemArtist::lessThan);
+                foreach (MusicLibraryItem *a, artists) {
+                    const MusicLibraryItemContainer *artist=static_cast<const MusicLibraryItemContainer *>(a);
+                    // Now sort all albums as they would appear in UI...
+                    QList<MusicLibraryItem *> artistAlbums=static_cast<const MusicLibraryItemContainer *>(artist)->childItems();
+                    qSort(artistAlbums.begin(), artistAlbums.end(), MusicLibraryItemAlbum::lessThan);
+                    foreach (MusicLibraryItem *i, artistAlbums) {
+                        const MusicLibraryItemContainer *album=static_cast<const MusicLibraryItemContainer *>(i);
+                        foreach (const MusicLibraryItem *song, static_cast<const MusicLibraryItemContainer *>(album)->childItems()) {
+                            if (MusicLibraryItem::Type_Song==song->itemType() && !devSongs[parent].contains(static_cast<const MusicLibraryItemSong*>(song)->song())) {
+                                devSongs[parent] << fixPath(static_cast<const MusicLibraryItemSong*>(song)->song(), base);
+                            }
                         }
                     }
                 }
@@ -541,9 +660,15 @@ void DevicesModel::deviceAdded(const QString &udi)
 
     Solid::Device device(udi);
     DBUG << "Solid device added udi:" << device.udi() << "product:" << device.product() << "vendor:" << device.vendor();
-    Solid::StorageAccess *ssa = device.as<Solid::StorageAccess>();
+    Solid::StorageAccess *ssa =0;
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    Solid::OpticalDisc * opt = device.as<Solid::OpticalDisc>();
 
-    if (ssa) {
+    if (opt && (opt->availableContent()&Solid::OpticalDisc::Audio)) {
+        DBUG << "device is audiocd";
+    } else
+    #endif
+    if ((ssa=device.as<Solid::StorageAccess>())) {
         if ((!device.parent().as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.parent().as<Solid::StorageDrive>()->bus()) &&
             (!device.as<Solid::StorageDrive>() || Solid::StorageDrive::Usb!=device.as<Solid::StorageDrive>()->bus())) {
             DBUG << "Found Solid::StorageAccess that is not usb, skipping";
@@ -567,6 +692,9 @@ void DevicesModel::deviceAdded(const QString &udi)
 
 void DevicesModel::addLocalDevice(const QString &udi)
 {
+    if (device(udi)) {
+        return;
+    }
     Device *dev=Device::create(this, udi);
     if (dev) {
         beginInsertRows(QModelIndex(), devices.count(), devices.count());
@@ -575,6 +703,12 @@ void DevicesModel::addLocalDevice(const QString &udi)
         connect(dev, SIGNAL(updating(const QString &, bool)), SLOT(deviceUpdating(const QString &, bool)));
         connect(dev, SIGNAL(error(const QString &)), SIGNAL(error(const QString &)));
         connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
+        #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+        if (Device::AudioCd==dev->devType()) {
+            connect(static_cast<AudioCdDevice *>(dev), SIGNAL(matches(const QString &, const QList<CdAlbum> &)),
+                    SIGNAL(matches(const QString &, const QList<CdAlbum> &)));
+        }
+        #endif
         updateItemMenu();
     }
 }
@@ -597,6 +731,7 @@ void DevicesModel::deviceRemoved(const QString &udi)
         devices.takeAt(idx)->deleteLater();
         endRemoveRows();
         updateItemMenu();
+        updateGenres();
     }
 }
 
@@ -614,10 +749,10 @@ void DevicesModel::accessibilityChanged(bool accessible, const QString &udi)
         }
     }
 }
-void DevicesModel::addRemoteDevice(const QString &coverFileName, const DeviceOptions &opts, RemoteFsDevice::Details details)
+void DevicesModel::addRemoteDevice(const DeviceOptions &opts, RemoteFsDevice::Details details)
 {
     #ifdef ENABLE_REMOTE_DEVICES
-    Device *dev=RemoteFsDevice::create(this, coverFileName, opts, details);
+    Device *dev=RemoteFsDevice::create(this, opts, details);
 
     if (dev) {
         beginInsertRows(QModelIndex(), devices.count(), devices.count());
@@ -628,17 +763,18 @@ void DevicesModel::addRemoteDevice(const QString &coverFileName, const DeviceOpt
         connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
         if (Device::RemoteFs==dev->devType()) {
             connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(udiChanged()), SLOT(remoteDeviceUdiChanged()));
+            connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(connectionStateHasChanged(QString, bool)),
+                    SLOT(removeDeviceConnectionStateChanged(QString, bool)));
         }
         updateItemMenu();
     }
     #else
-    Q_UNUSED(coverFileName)
     Q_UNUSED(opts)
     Q_UNUSED(details)
     #endif
 }
 
-void DevicesModel::removeRemoteDevice(const QString &udi)
+void DevicesModel::removeRemoteDevice(const QString &udi, bool removeFromConfig)
 {
     #ifdef ENABLE_REMOTE_DEVICES
     int idx=indexOf(udi);
@@ -654,11 +790,16 @@ void DevicesModel::removeRemoteDevice(const QString &udi)
         devices.takeAt(idx);
         endRemoveRows();
         updateItemMenu();
-        // Remove will stop device, and delete it
-        RemoteFsDevice::remove(dev);
+        updateGenres();
+        RemoteFsDevice *rfs=qobject_cast<RemoteFsDevice *>(dev);
+        if (rfs) {
+            // Destroy will stop device, and delete it (via deleteLater())
+            rfs->destroy(removeFromConfig);
+        }
     }
     #else
     Q_UNUSED(udi)
+    Q_UNUSED(removeFromConfig)
     #endif
 }
 
@@ -669,11 +810,27 @@ void DevicesModel::remoteDeviceUdiChanged()
     #endif
 }
 
+void DevicesModel::removeDeviceConnectionStateChanged(const QString &udi, bool state)
+{
+    #ifdef ENABLE_REMOTE_DEVICES
+    if (!state) {
+        int idx=indexOf(udi);
+        if (idx<0) {
+            return;
+        }
+        updateGenres();
+    }
+    #else
+    Q_UNUSED(udi)
+    Q_UNUSED(state)
+    #endif
+}
+
 void DevicesModel::mountsChanged()
 {
     #ifdef ENABLE_REMOTE_DEVICES
     foreach (Device *dev, devices) {
-        if (Device::RemoteFs==dev->devType()) {
+        if (Device::RemoteFs==dev->devType() && ((RemoteFsDevice *)dev)->getDetails().isLocalFile()) {
             if (0==dev->childCount()) {
                 ((RemoteFsDevice *)dev)->load();
             } else if (!dev->isConnected()) {
@@ -736,6 +893,23 @@ void DevicesModel::loadLocal()
         }
     }
 
+    #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
+    deviceList = Solid::Device::listFromType(Solid::DeviceInterface::OpticalDisc);
+    foreach (const Solid::Device &device, deviceList) {
+        if (existingUdis.contains(device.udi())) {
+            existingUdis.remove(device.udi());
+            continue;
+        }
+        DBUG << "Solid::OpticalDisc with udi:" << device.udi() << "product:" << device.product() << "vendor:" << device.vendor();
+        const Solid::OpticalDisc * opt = device.as<Solid::OpticalDisc>();
+        if (opt && (opt->availableContent()&Solid::OpticalDisc::Audio)) {
+            addLocalDevice(device.udi());
+        } else {
+            DBUG << "Solid::OpticalDisc that is not audio, skipping";
+        }
+    }
+    #endif
+
     // Remove any previous MTP/UMS devices that were not listed above.
     // This is to fix BUG:127
     foreach (const QString &udi, existingUdis) {
@@ -756,6 +930,8 @@ void DevicesModel::loadRemote()
             connect(dev, SIGNAL(cover(const Song &, const QImage &)), SLOT(setCover(const Song &, const QImage &)));
             if (Device::RemoteFs==dev->devType()) {
                 connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(udiChanged()), SLOT(remoteDeviceUdiChanged()));
+                connect(static_cast<RemoteFsDevice *>(dev), SIGNAL(connectionStateHasChanged(QString, bool)),
+                        SLOT(removeDeviceConnectionStateChanged(QString, bool)));
             }
         }
         endInsertRows();
@@ -817,7 +993,9 @@ void DevicesModel::updateItemMenu()
         QMap<QString, const Device *> items;
 
         foreach (const Device *d, devices) {
-            items.insert(d->data(), d);
+            if (Device::AudioCd!=d->devType()) {
+                items.insert(d->data(), d);
+            }
         }
 
         QStringList keys=items.keys();
@@ -825,7 +1003,9 @@ void DevicesModel::updateItemMenu()
 
         foreach (const QString &k, keys) {
             const Device *d=items[k];
-            itemMenu->addAction(QIcon::fromTheme(d->icon()), k, this, SLOT(emitAddToDevice()))->setData(d->udi());
+            QAction *act=itemMenu->addAction(QIcon::fromTheme(d->icon()), k, this, SLOT(emitAddToDevice()));
+            act->setData(d->udi());
+            Action::initIcon(act);
         }
     }
 }
@@ -840,7 +1020,7 @@ QMimeData * DevicesModel::mimeData(const QModelIndexList &indexes) const
             paths.append(HttpServer::self()->encodeUrl(s));
         }
     } else {
-        paths=filenames(indexes, true, true);
+        paths=playableUrls(indexes);
     }
 
     if (!paths.isEmpty()) {
