@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -24,9 +24,9 @@
  * along with QtMPC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <QtCore/QModelIndex>
-#include <QtCore/QMimeData>
-#include <QtGui/QMenu>
+#include <QModelIndex>
+#include <QMimeData>
+#include <QMenu>
 #include "playlistsmodel.h"
 #include "playlistsproxymodel.h"
 #include "itemview.h"
@@ -42,6 +42,7 @@ K_GLOBAL_STATIC(PlaylistsModel, instance)
 #include "mpdconnection.h"
 #include "playqueuemodel.h"
 #include "icons.h"
+#include "gtkstyle.h"
 
 PlaylistsModel * PlaylistsModel::self()
 {
@@ -57,7 +58,7 @@ PlaylistsModel * PlaylistsModel::self()
 }
 
 PlaylistsModel::PlaylistsModel(QObject *parent)
-    : QAbstractItemModel(parent)
+    : ActionModel(parent)
     , itemMenu(0)
     , dropAdjust(0)
 {
@@ -73,6 +74,9 @@ PlaylistsModel::PlaylistsModel(QObject *parent)
     connect(this, SIGNAL(playlistInfo(const QString &)), MPDConnection::self(), SLOT(playlistInfo(const QString &)));
     connect(this, SIGNAL(addToPlaylist(const QString &, const QStringList, quint32, quint32)), MPDConnection::self(), SLOT(addToPlaylist(const QString &, const QStringList, quint32, quint32)));
     connect(this, SIGNAL(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)), MPDConnection::self(), SLOT(moveInPlaylist(const QString &, const QList<quint32> &, quint32, quint32)));
+    newAction=new QAction(Icon("document-new"), i18n("New Playlist..."), this);
+    connect(newAction, SIGNAL(triggered(bool)), this, SIGNAL(addToNew()));
+    Action::initIcon(newAction);
     updateItemMenu();
 }
 
@@ -203,7 +207,8 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             #else
             return QTP_TRACKS_DURATION_STR(pl->songs.count(), Song::formattedTime(pl->totalTime()));
             #endif
-        default: break;
+        default:
+            return ActionModel::data(index, role);
         }
     } else {
         SongItem *s=static_cast<SongItem *>(item);
@@ -282,6 +287,8 @@ QVariant PlaylistsModel::data(const QModelIndex &index, int role) const
             return s->title.isEmpty() ? s->file : s->title;
         case ItemView::Role_SubText:
             return s->artist+QLatin1String(" - ")+s->album;
+        default:
+            return ActionModel::data(index, role);
         }
     }
 
@@ -390,6 +397,10 @@ bool PlaylistsModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
     if (row<0) {
         row=parent.row();
     }
+
+    if (origRow<0 && dropAdjust) { // Unlike the playqueue, we 'copy' items, so need to adjust 1 more place...
+        dropAdjust++;
+    }
     row+=dropAdjust;
 
     Q_UNUSED(col)
@@ -659,19 +670,28 @@ void PlaylistsModel::removedFromPlaylist(const QString &name, const QList<quint3
 void PlaylistsModel::movedInPlaylist(const QString &name, const QList<quint32> &idx, quint32 pos)
 {
     PlaylistItem *pl=0;
-    if (!(pl=getPlaylist(name)) || items.count()>pl->songs.count()) {
+    if (!(pl=getPlaylist(name)) || idx.count()>pl->songs.count()) {
         emit listPlaylists();
         return;
     }
 
+    foreach (quint32 i, idx) {
+        if (i>=(quint32)pl->songs.count()) {
+            emit listPlaylists();
+            return;
+        }
+    }
+
     QList<quint32> indexes=idx;
     QModelIndex parent=createIndex(items.indexOf(pl), 0, pl);
     while (indexes.count()) {
         quint32 from=indexes.takeLast();
-        beginMoveRows(parent, from, from, parent, pos>from ? pos+1 : pos);
-        SongItem *si=pl->songs.takeAt(from);
-        pl->songs.insert(pos, si);
-        endMoveRows();
+        if (from!=pos) {
+            beginMoveRows(parent, from, from, parent, pos>from && pos<(quint32)pl->songs.count() ? pos+1 : pos);
+            SongItem *si=pl->songs.takeAt(from);
+            pl->songs.insert(pos, si);
+            endMoveRows();
+        }
     }
     emit updated(parent);
 }
@@ -702,8 +722,7 @@ void PlaylistsModel::updateItemMenu()
     }
 
     itemMenu->clear();
-    itemMenu->addAction(Icon("document-new"), i18n("New Playlist..."), this, SIGNAL(addToNew()));
-
+    itemMenu->addAction(newAction);
     QStringList names;
     foreach (const PlaylistItem *p, items) {
         names << p->name;
@@ -739,12 +758,15 @@ void PlaylistsModel::clearPlaylists()
 
 void PlaylistsModel::updateGenreList()
 {
-    QSet<QString> genres;
+    QSet<QString> newGenres;
     foreach (PlaylistItem *p, items) {
-        genres+=p->genres;
+        newGenres+=p->genres;
     }
 
-    emit updateGenres(genres);
+    if (newGenres!=plGenres) {
+        plGenres=newGenres;
+        emit updateGenres(plGenres);
+    }
 }
 
 quint32 PlaylistsModel::allocateKey()
