@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -33,14 +33,14 @@
 #include "localize.h"
 #include "messagebox.h"
 #include "jobcontroller.h"
-#include <QtGui/QTreeWidget>
-#include <QtGui/QLabel>
-#include <QtGui/QProgressBar>
-#include <QtGui/QBoxLayout>
-#include <QtGui/QHeaderView>
-#include <QtGui/QCloseEvent>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QEventLoop>
+#include <QTreeWidget>
+#include <QLabel>
+#include <QProgressBar>
+#include <QBoxLayout>
+#include <QHeaderView>
+#include <QCloseEvent>
+#include <QCoreApplication>
+#include <QEventLoop>
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KLocale>
 #endif
@@ -80,8 +80,30 @@ int RgDialog::instanceCount()
     return iCount;
 }
 
+#if QT_VERSION < 0x050000
+static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
+{
+    hdr->setResizeMode(idx, mode);
+}
+
+static inline void setResizeMode(QHeaderView *hdr, QHeaderView::ResizeMode mode)
+{
+    hdr->setResizeMode(mode);
+}
+#else
+static inline void setResizeMode(QHeaderView *hdr, int idx, QHeaderView::ResizeMode mode)
+{
+    hdr->setSectionResizeMode(idx, mode);
+}
+
+static inline void setResizeMode(QHeaderView *hdr, QHeaderView::ResizeMode mode)
+{
+    hdr->setSectionResizeMode(mode);
+}
+#endif
+
 RgDialog::RgDialog(QWidget *parent)
-    : Dialog(parent)
+    : Dialog(parent, "RgDialog")
     , state(State_Idle)
     , totalToScan(0)
     , tagReader(0)
@@ -90,8 +112,8 @@ RgDialog::RgDialog(QWidget *parent)
     setButtons(User1|Ok|Cancel);
     setCaption(i18n("ReplayGain"));
     setAttribute(Qt::WA_DeleteOnClose);
-    QWidget *mainWidet = new QWidget(this);
-    QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, mainWidet);
+    QWidget *mainWidget = new QWidget(this);
+    QBoxLayout *layout=new QBoxLayout(QBoxLayout::TopToBottom, mainWidget);
     view = new QTreeWidget(this);
     statusLabel = new QLabel(this);
     statusLabel->setVisible(false);
@@ -99,6 +121,7 @@ RgDialog::RgDialog(QWidget *parent)
     progress->setVisible(false);
     view->setRootIsDecorated(false);
     view->setAllColumnsShowFocus(true);
+    view->setAlternatingRowColors(true);
     QTreeWidgetItem *hdr = view->headerItem();
     hdr->setText(COL_ARTIST, i18n("Artist"));
     hdr->setText(COL_ALBUM, i18n("Album"));
@@ -109,19 +132,20 @@ RgDialog::RgDialog(QWidget *parent)
     hdr->setText(COL_TRACKPEAK, i18n("Track Peak"));
 
     QHeaderView *hv=view->header();
-    hv->setResizeMode(COL_ARTIST, QHeaderView::ResizeToContents);
-    hv->setResizeMode(COL_ALBUM, QHeaderView::ResizeToContents);
-    hv->setResizeMode(COL_TITLE, QHeaderView::Stretch);
-    hv->setResizeMode(COL_ALBUMGAIN, QHeaderView::Fixed);
-    hv->setResizeMode(COL_TRACKGAIN, QHeaderView::Fixed);
-    hv->setResizeMode(COL_ALBUMPEAK, QHeaderView::Fixed);
-    hv->setResizeMode(COL_TRACKPEAK, QHeaderView::Fixed);
+    setResizeMode(hv, COL_ARTIST, QHeaderView::ResizeToContents);
+    setResizeMode(hv, COL_ALBUM, QHeaderView::ResizeToContents);
+    setResizeMode(hv, COL_TITLE, QHeaderView::Stretch);
+    setResizeMode(hv, COL_ALBUMGAIN, QHeaderView::Fixed);
+    setResizeMode(hv, COL_TRACKGAIN, QHeaderView::Fixed);
+    setResizeMode(hv, COL_ALBUMPEAK, QHeaderView::Fixed);
+    setResizeMode(hv, COL_TRACKPEAK, QHeaderView::Fixed);
     hv->setStretchLastSection(false);
 
+    layout->setMargin(0);
     layout->addWidget(view);
     layout->addWidget(statusLabel);
     layout->addWidget(progress);
-    setMainWidget(mainWidet);
+    setMainWidget(mainWidget);
     #ifdef ENABLE_KDE_SUPPORT
     setButtonGuiItem(Ok, KStandardGuiItem::save());
     setButtonGuiItem(Cancel, KStandardGuiItem::close());
@@ -393,11 +417,7 @@ void RgDialog::saveTags()
     }
 
     if (failed.count()) {
-        #ifdef ENABLE_KDE_SUPPORT
-        MessageBox::errorList(this, i18n("Failed to update the tags of the following tracks:"), failed);
-        #else
-        MessageBox::errorList(this, QObject::tr("Failed to update the tags of some tracks."), failed);
-        #endif
+        MessageBox::errorListEx(this, i18n("Failed to update the tags of the following tracks:"), failed);
     }
 }
 
@@ -608,7 +628,7 @@ void RgDialog::tagReaderDone()
         return;
     }
 
-    t->stop();
+    JobController::self()->finishedWith(t);
     tagReader=0;
 
     state=State_Idle;
