@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2012 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2013 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -27,20 +27,20 @@
 #include "covers.h"
 #include "proxymodel.h"
 #include "actionitemdelegate.h"
+#include "actionmodel.h"
 #include "localize.h"
 #include "icon.h"
 #include "config.h"
 #include "gtkstyle.h"
-#include <QtGui/QToolButton>
-#include <QtGui/QStyle>
-#include <QtGui/QStyleOptionViewItem>
-#include <QtGui/QPainter>
-#include <QtGui/QAction>
-#include <QtCore/QTimer>
+#include "spinner.h"
+#include <QToolButton>
+#include <QStyle>
+#include <QStyleOptionViewItem>
+#include <QPainter>
+#include <QAction>
+#include <QTimer>
 #ifdef ENABLE_KDE_SUPPORT
 #include <KDE/KGlobalSettings>
-#include <KDE/KPixmapSequence>
-#include <KDE/KPixmapSequenceOverlayPainter>
 #endif
 
 static bool forceSingleClick=true;
@@ -67,26 +67,9 @@ void ItemView::setup()
     }
 }
 
-EscapeKeyEventHandler::EscapeKeyEventHandler(QAbstractItemView *v, QAction *a)
-    : QObject(v)
-    , view(v)
-    , act(a)
-{
-}
-
-bool EscapeKeyEventHandler::eventFilter(QObject *obj, QEvent *event)
-{
-    if (view->hasFocus() && QEvent::KeyRelease==event->type() && static_cast<QKeyEvent *>(event)->key()==Qt::Key_Escape) {
-        if (act->isEnabled()) {
-            act->trigger();
-        }
-        return true;
-    }
-    return QObject::eventFilter(obj, event);
-}
-
-MouseEventHandler::MouseEventHandler(QAbstractItemView *v)
+ViewEventHandler::ViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v)
     : QObject(v)
+    , delegate(d)
     , view(v)
 {
 }
@@ -94,90 +77,37 @@ MouseEventHandler::MouseEventHandler(QAbstractItemView *v)
 // HACK time. For some reason, IconView is not always re-drawn when mouse leaves the view.
 // We sometimes get an item that is left in the mouse-over state. So, work-around this by
 // keeping track of when mouse is over listview.
-static QWidget *mouseWidget=0;
-bool MouseEventHandler::eventFilter(QObject *obj, QEvent *event)
+bool ViewEventHandler::eventFilter(QObject *obj, QEvent *event)
 {
-    if (QEvent::Enter==event->type()) {
-        mouseWidget=view;
-        view->viewport()->update();
-    } else if (QEvent::Leave==event->type()) {
-        mouseWidget=0;
-        view->viewport()->update();
+    if (delegate) {
+        if (QEvent::Enter==event->type()) {
+            delegate->setUnderMouse(true);
+            view->viewport()->update();
+        } else if (QEvent::Leave==event->type()) {
+            delegate->setUnderMouse(false);
+            view->viewport()->update();
+        }
     }
     return QObject::eventFilter(obj, event);
 }
 
-#ifndef ENABLE_KDE_SUPPORT
-Spinner::Spinner()
-    : QWidget(0)
-    , timer(0)
-    , value(0)
+ListViewEventHandler::ListViewEventHandler(ActionItemDelegate *d, QAbstractItemView *v, QAction *a)
+    : ViewEventHandler(d, v)
+    , act(a)
 {
-    static const int constSize=24;
-    setVisible(false);
-    setMinimumSize(constSize, constSize);
-    setMaximumSize(constSize, constSize);
 }
 
-void Spinner::start()
+bool ListViewEventHandler::eventFilter(QObject *obj, QEvent *event)
 {
-    value=0;
-    setVisible(true);
-    setPosition();
-    if (!timer) {
-        timer=new QTimer(this);
-        connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));
+    if (view->hasFocus() && QEvent::KeyRelease==event->type() && static_cast<QKeyEvent *>(event)->key()==Qt::Key_Escape) {
+        if (act->isEnabled()) {
+            act->trigger();
+        }
+        return true;
     }
-    timer->start(100);
+    return ViewEventHandler::eventFilter(obj, event);
 }
 
-void Spinner::stop()
-{
-    setVisible(false);
-    if (timer) {
-        timer->stop();
-    }
-}
-
-static const int constSpinnerSteps=64;
-
-void Spinner::paintEvent(QPaintEvent *event)
-{
-    static const int constParts=8;
-    QPainter p(this);
-    QRectF rectangle(1.5, 1.5, size().width()-3, size().height()-3);
-    QColor col(palette().color(QPalette::Text));
-    p.setRenderHint(QPainter::Antialiasing, true);
-    p.setClipRect(event->rect());
-    double size=(360*16)/(2.0*constParts);
-    for (int i=0; i<constParts; ++i) {
-        col.setAlphaF((constParts-i)/(1.0*constParts));
-        p.setPen(QPen(col, 2));
-        p.drawArc(rectangle, (((constSpinnerSteps-value)*1.0)/(constSpinnerSteps*1.0)*360*16)+(i*2.0*size), size);
-    }
-    p.end();
-}
-
-void Spinner::timeout()
-{
-    setPosition();
-    update();
-    if (++value>=constSpinnerSteps) {
-        value=0;
-    }
-}
-
-void Spinner::setPosition()
-{
-    QPoint current=pos();
-    QPoint desired=QPoint(parentWidget()->size().width()-(size().width()+4), 4);
-
-    if (current!=desired) {
-        move(desired);
-    }
-}
-#endif
-
 static const int constImageSize=22;
 static const int constDevImageSize=32;
 
@@ -189,8 +119,8 @@ static inline double subTextAlpha(bool selected)
 class ListDelegate : public ActionItemDelegate
 {
 public:
-    ListDelegate(ListView *v, QAbstractItemView *p, QAction *a1, QAction *a2, QAction *t, int actionLevel)
-        : ActionItemDelegate(p, a1, a2, t, actionLevel)
+    ListDelegate(ListView *v, QAbstractItemView *p)
+        : ActionItemDelegate(p)
         , view(v)
     {
     }
@@ -237,12 +167,13 @@ public:
         QStyleOptionViewItemV4 opt(option);
         opt.showDecorationSelected=true;
 
-        if (view!=mouseWidget) {
+        if (!underMouse) {
             if (mouseOver && !selected) {
                 drawBgnd=false;
             }
             mouseOver=false;
         }
+
         if (drawBgnd) {
             if (mouseOver && gtk) {
                 GtkStyle::drawSelection(opt, painter, selected ? 0.75 : 0.25);
@@ -265,9 +196,15 @@ public:
             image = index.data(Qt::DecorationRole);
         }
 
-        QPixmap pix = QVariant::Pixmap==image.type() ? image.value<QPixmap>() : image.value<QIcon>().pixmap(listDecorationSize, listDecorationSize);
+        QPixmap pix = QVariant::Pixmap==image.type()
+                        ? image.value<QPixmap>()
+                        : QVariant::Image==image.type()
+                            ? QPixmap::fromImage(image.value<QImage>().scaled(listDecorationSize, listDecorationSize,
+                                                                              Qt::KeepAspectRatio, Qt::SmoothTransformation))
+                            : image.value<QIcon>().pixmap(listDecorationSize, listDecorationSize);
         bool oneLine = childText.isEmpty();
         bool iconMode = view && QListView::IconMode==view->viewMode();
+        ActionPos actionPos = iconMode ? AP_VTop : AP_HMiddle;
         bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
 
         painter->save();
@@ -281,7 +218,7 @@ public:
             r.adjust(2, 0, 0, -(textHeight+4));
         }
 
-        if (iconMode) {
+        if (AP_VTop==actionPos) {
             r.adjust(constBorder, constBorder, -constBorder, -constBorder);
             r2=r;
         } else {
@@ -289,8 +226,37 @@ public:
         }
         if (!pix.isNull()) {
             int adjust=qMax(pix.width(), pix.height());
-            if (iconMode) {
-                painter->drawPixmap(r.x()+((r.width()-pix.width())/2), r.y(), pix.width(), pix.height(), pix);
+            if (AP_VTop==actionPos) {
+                int xpos=r.x()+((r.width()-pix.width())/2);
+                painter->drawPixmap(xpos, r.y(), pix.width(), pix.height(), pix);
+                QColor color(option.palette.color(active ? QPalette::Active : QPalette::Inactive, QPalette::Text));
+                double alphas[]={0.25, 0.125, 0.061};
+                QRect border(xpos, r.y(), pix.width(), pix.height());
+                QRect shadow(border);
+                for (int i=0; i<3; ++i) {
+                    shadow.adjust(1, 1, 1, 1);
+                    color.setAlphaF(alphas[i]);
+                    painter->setPen(color);
+                    painter->drawLine(shadow.bottomLeft()+QPoint(i+1, 0),
+                                      shadow.bottomRight()+QPoint(-((i*2)+2), 0));
+                    painter->drawLine(shadow.bottomRight()+QPoint(0, -((i*2)+2)),
+                                      shadow.topRight()+QPoint(0, i+1));
+                    if (1==i) {
+                        painter->drawPoint(shadow.bottomRight()-QPoint(2, 1));
+                        painter->drawPoint(shadow.bottomRight()-QPoint(1, 2));
+                        painter->drawPoint(shadow.bottomLeft()-QPoint(1, 1));
+                        painter->drawPoint(shadow.topRight()-QPoint(1, 1));
+                    } else if (2==i) {
+                        painter->drawPoint(shadow.bottomRight()-QPoint(4, 1));
+                        painter->drawPoint(shadow.bottomRight()-QPoint(1, 4));
+                        painter->drawPoint(shadow.bottomLeft()-QPoint(0, 1));
+                        painter->drawPoint(shadow.topRight()-QPoint(1, 0));
+                        painter->drawPoint(shadow.bottomRight()-QPoint(2, 2));
+                    }
+                }
+                color.setAlphaF(0.4);
+                painter->setPen(color);
+                painter->drawRect(border.adjusted(0, 0, -1, -1));
                 r.adjust(0, adjust+3, 0, -3);
             } else {
                 if (rtl) {
@@ -303,14 +269,15 @@ public:
             }
         }
 
-        if (!mouseOver && hasActions(index, actLevel)) {
-            drawIcons(painter, iconMode ? r2 : r, false, rtl, iconMode, index);
+        if (!mouseOver) {
+            drawIcons(painter, AP_VTop==actionPos ? r2 : r, false, rtl, actionPos, index);
         }
 
         QRect textRect;
         QColor color(option.palette.color(active ? QPalette::Active : QPalette::Inactive, selected ? QPalette::HighlightedText : QPalette::Text));
-        QTextOption textOpt(iconMode ? Qt::AlignHCenter|Qt::AlignVCenter : Qt::AlignVCenter);
+        QTextOption textOpt(AP_VTop==actionPos ? Qt::AlignHCenter|Qt::AlignVCenter : Qt::AlignVCenter);
 
+        textOpt.setWrapMode(QTextOption::NoWrap);
         if (oneLine) {
             textRect=QRect(r.x(), r.y()+((r.height()-textHeight)/2), r.width(), textHeight);
             text = textMetrics.elidedText(text, Qt::ElideRight, textRect.width(), QPalette::WindowText);
@@ -344,9 +311,14 @@ public:
             QStyleOptionProgressBar opt;
             double capacity=index.data(ItemView::Role_Capacity).toDouble();
 
+            if (capacity<0.0) {
+                capacity=0.0;
+            } else if (capacity>1.0) {
+                capacity=1.0;
+            }
             opt.minimum=0;
             opt.maximum=1000;
-            opt.progress=capacity<0 ? 0 : (index.data(ItemView::Role_Capacity).toDouble()*1000);
+            opt.progress=capacity*1000;
             opt.textVisible=true;
             opt.text=capacityText;
             opt.rect=QRect(r2.x()+4, r2.bottom()-(textHeight+4), r2.width()-8, textHeight);
@@ -358,22 +330,23 @@ public:
             QApplication::style()->drawControl(QStyle::CE_ProgressBar, &opt, painter, 0L);
         }
 
-        if (drawBgnd && mouseOver && hasActions(index, actLevel)) {
-            drawIcons(painter, iconMode ? r2 : r, true, rtl, iconMode, index);
+        if (drawBgnd && mouseOver) {
+            drawIcons(painter, AP_VTop==actionPos ? r2 : r, true, rtl, actionPos, index);
         }
 
         painter->restore();
     }
 
-private:
+protected:
     ListView *view;
 };
 
 class TreeDelegate : public ListDelegate
 {
 public:
-    TreeDelegate(QAbstractItemView *p, QAction *a1, QAction *a2, QAction *t, int actionLevel)
-        : ListDelegate(0, p, a1, a2, t, actionLevel)
+    TreeDelegate(QAbstractItemView *p)
+        : ListDelegate(0, p)
+        , simpleStyle(true)
     {
     }
 
@@ -383,7 +356,7 @@ public:
 
     QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
     {
-        if (!index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
+        if (!simpleStyle || !index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
             return ListDelegate::sizeHint(option, index);
         }
 
@@ -398,7 +371,7 @@ public:
             return;
         }
 
-        if(!index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
+        if (!simpleStyle || !index.data(ItemView::Role_CapacityText).toString().isEmpty()) {
             ListDelegate::paint(painter, option, index);
             return;
         }
@@ -467,12 +440,47 @@ public:
             }
         }
 
-        if ((option.state & QStyle::State_MouseOver) && hasActions(index, actLevel)) {
-            drawIcons(painter, option.rect, true, rtl, false, index);
+        if ((option.state & QStyle::State_MouseOver)) {
+            drawIcons(painter, option.rect, true, rtl, AP_HMiddle, index);
         }
     }
+
+    void setSimple(bool s) { simpleStyle=s; }
+
+    bool simpleStyle;
 };
 
+ItemView::Mode ItemView::toMode(const QString &str)
+{
+    for (int i=0; i<Mode_Count; ++i) {
+        if (modeStr((Mode)i)==str) {
+            return (Mode)i;
+        }
+    }
+
+    // Older versions of Cantata saved mode as an integer!!!
+    int i=str.toInt();
+    switch (i) {
+        default:
+        case 0: return Mode_SimpleTree;
+        case 1: return Mode_List;
+        case 2: return Mode_IconTop;
+        case 3: return Mode_GroupedTree;
+    }
+}
+
+QString ItemView::modeStr(Mode m)
+{
+    switch (m) {
+    default:
+    case Mode_SimpleTree:   return QLatin1String("simpletree");
+    case Mode_DetailedTree: return QLatin1String("detailedtree");
+    case Mode_List:         return QLatin1String("list");
+    case Mode_IconTop:      return QLatin1String("icontop");
+    case Mode_GroupedTree:  return QLatin1String("grouped");
+    }
+}
+
 void ItemView::setForceSingleClick(bool v)
 {
     forceSingleClick=v;
@@ -487,20 +495,16 @@ ItemView::ItemView(QWidget *p)
     : QWidget(p)
     , searchTimer(0)
     , itemModel(0)
-    , actLevel(-1)
-    , act1(0)
-    , act2(0)
-    , toggle(0)
     , currentLevel(0)
-    , mode(Mode_Tree)
+    , mode(Mode_SimpleTree)
     , groupedView(0)
-    , spinnerActive(false)
     , spinner(0)
 {
     setupUi(this);
     backAction = new QAction(i18n("Back"), this);
     backAction->setIcon(Icon("go-previous"));
     backButton->setDefaultAction(backAction);
+    //backButton->setMaximumHeight(listSearch->sizeHint().height());
     backAction->setShortcut(Qt::Key_Escape);
     listView->addAction(backAction);
     listView->addDefaultAction(backAction);
@@ -509,45 +513,15 @@ ItemView::ItemView(QWidget *p)
     listView->addAction(sep);
     Icon::init(backButton);
     treeView->setPageDefaults();
+    // Some styles, eg Cleanlooks/Plastique require that we explicitly set mouse tracking on the treeview.
+    treeView->setAttribute(Qt::WA_MouseTracking);
     iconGridSize=listGridSize=listView->gridSize();
-    listView->installEventFilter(new EscapeKeyEventHandler(listView, backAction));
-    listView->installEventFilter(new MouseEventHandler(listView));
-}
-
-ItemView::~ItemView()
-{
-}
-
-void ItemView::allowGroupedView()
-{
-    if (!groupedView) {
-        groupedView=new GroupedView(stackedWidget);
-        groupedView->setAutoExpand(false);
-        treeLayout->addWidget(groupedView);
-        connect(groupedView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
-        if (SINGLE_CLICK) {
-            connect(groupedView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
-        }
-        connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
-        connect(groupedView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
-    }
-}
-
-void ItemView::init(QAction *a1, QAction *a2, QAction *t, int actionLevel)
-{
-    if (act1 || act2 || toggle) {
-        return;
-    }
-
-    act1=a1;
-    act2=a2;
-    toggle=t;
-    actLevel=actionLevel;
-    listView->setItemDelegate(new ListDelegate(listView, listView, a1, a2, toggle, actionLevel));
-    treeView->setItemDelegate(new TreeDelegate(treeView, a1, a2, toggle, actionLevel));
-    if (groupedView) {
-        groupedView->init(0, 0, 0, 0); // No actions in grouped view :-(
-    }
+    ListDelegate *ld=new ListDelegate(listView, listView);
+    TreeDelegate *td=new TreeDelegate(treeView);
+    listView->setItemDelegate(ld);
+    treeView->setItemDelegate(td);
+    listView->installEventFilter(new ListViewEventHandler(ld, listView, backAction));
+    treeView->installEventFilter(new ViewEventHandler(td, treeView));
     connect(treeSearch, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
     connect(treeSearch, SIGNAL(textChanged(const QString)), this, SLOT(delaySearchItems()));
     connect(listSearch, SIGNAL(returnPressed()), this, SLOT(delaySearchItems()));
@@ -563,8 +537,29 @@ void ItemView::init(QAction *a1, QAction *a2, QAction *t, int actionLevel)
     connect(listView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
     connect(listView, SIGNAL(clicked(const QModelIndex &)),  this, SLOT(itemClicked(const QModelIndex &)));
     connect(backAction, SIGNAL(triggered(bool)), this, SLOT(backActivated()));
-    mode=Mode_List;
-    setMode(Mode_Tree);
+}
+
+ItemView::~ItemView()
+{
+}
+
+void ItemView::allowGroupedView()
+{
+    if (!groupedView) {
+        groupedView=new GroupedView(stackedWidget);
+        // Some styles, eg Cleanlooks/Plastique require that we explicitly set mouse tracking on the treeview.
+        groupedView->setAttribute(Qt::WA_MouseTracking, true);
+        groupedView->installEventFilter(new ViewEventHandler(qobject_cast<ActionItemDelegate *>(groupedView->itemDelegate()), groupedView));
+        groupedView->setAutoExpand(false);
+        groupedView->setMultiLevel(true);
+        treeLayout->addWidget(groupedView);
+        connect(groupedView, SIGNAL(itemsSelected(bool)), this, SIGNAL(itemsSelected(bool)));
+        if (SINGLE_CLICK) {
+            connect(groupedView, SIGNAL(activated(const QModelIndex &)), this, SLOT(itemActivated(const QModelIndex &)));
+        }
+        connect(groupedView, SIGNAL(doubleClicked(const QModelIndex &)), this, SIGNAL(doubleClicked(const QModelIndex &)));
+        connect(groupedView, SIGNAL(clicked(const QModelIndex &)), this, SLOT(itemClicked(const QModelIndex &)));
+    }
 }
 
 void ItemView::addAction(QAction *act)
@@ -579,7 +574,7 @@ void ItemView::addAction(QAction *act)
 void ItemView::setMode(Mode m)
 {
     if (m<0 || m>Mode_GroupedTree || (Mode_GroupedTree==m && !groupedView)) {
-        m=Mode_Tree;
+        m=Mode_SimpleTree;
     }
 
     if (m==mode) {
@@ -589,7 +584,7 @@ void ItemView::setMode(Mode m)
     mode=m;
     treeSearch->setText(QString());
     listSearch->setText(QString());
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         treeView->setModel(itemModel);
         listView->setModel(0);
         if (groupedView) {
@@ -597,7 +592,9 @@ void ItemView::setMode(Mode m)
             groupedView->setModel(0);
         }
         treeView->setHidden(false);
+        ((TreeDelegate *)treeView->itemDelegate())->setSimple(Mode_SimpleTree==mode);
         itemModel->setRootIndex(QModelIndex());
+        treeView->reset();
     } else if (Mode_GroupedTree==mode) {
         treeView->setModel(0);
         listView->setModel(0);
@@ -614,6 +611,7 @@ void ItemView::setMode(Mode m)
         setLevel(0);
         listView->setRootIndex(QModelIndex());
         itemModel->setRootIndex(QModelIndex());
+        emit rootIndexSet(QModelIndex());
         if (Mode_IconTop!=mode) {
             listView->setGridSize(listGridSize);
             listView->setViewMode(QListView::ListMode);
@@ -623,16 +621,10 @@ void ItemView::setMode(Mode m)
         }
     }
 
-    if (Mode_GroupedTree==mode) {
-        connect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), groupedView, SLOT(coverRetrieved(const Song &)));
-    } else if (groupedView) {
-        disconnect(Covers::self(), SIGNAL(coverRetrieved(const Song &)), groupedView, SLOT(coverRetrieved(const Song &)));
-    }
-
-    stackedWidget->setCurrentIndex(Mode_Tree==mode || Mode_GroupedTree==mode ? 0 : 1);
+    stackedWidget->setCurrentIndex(Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode ? 0 : 1);
     if (spinner) {
         spinner->setWidget(view()->viewport());
-        if (spinnerActive) {
+        if (spinner->isActive()) {
             spinner->start();
         }
     }
@@ -650,7 +642,7 @@ void ItemView::hideBackButton()
 
 QModelIndexList ItemView::selectedIndexes() const
 {
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         return treeView->selectedIndexes();
     } else if(Mode_GroupedTree==mode) {
         return groupedView->selectedIndexes();
@@ -699,7 +691,7 @@ void ItemView::setLevel(int l, bool haveChildren)
 
 QAbstractItemView * ItemView::view() const
 {
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         return treeView;
     } else if(Mode_GroupedTree==mode) {
         return groupedView;
@@ -710,18 +702,23 @@ QAbstractItemView * ItemView::view() const
 
 void ItemView::setModel(ProxyModel *m)
 {
+    bool needtToInit=!itemModel;
     itemModel=m;
+    if (needtToInit) {
+        mode=Mode_List;
+        setMode(Mode_SimpleTree);
+    }
     view()->setModel(m);
 }
 
 QString ItemView::searchText() const
 {
-    return Mode_Tree==mode || Mode_GroupedTree==mode ? treeSearch->text() : listSearch->text();
+    return Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode ? treeSearch->text() : listSearch->text();
 }
 
 void ItemView::clearSearchText()
 {
-    return Mode_Tree==mode || Mode_GroupedTree==mode ? treeSearch->setText(QString()) : listSearch->setText(QString());
+    return Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode ? treeSearch->setText(QString()) : listSearch->setText(QString());
 }
 
 void ItemView::setTopText(const QString &text)
@@ -775,7 +772,7 @@ void ItemView::setGridSize(const QSize &sz)
 
 void ItemView::update()
 {
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         treeView->update();
     } else if (Mode_GroupedTree==mode) {
         groupedView->update();
@@ -795,7 +792,7 @@ void ItemView::setDeleteAction(QAction *act)
 
 void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
 {
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         QModelIndex i=idx;
         while (i.isValid()) {
             treeView->setExpanded(i, true);
@@ -817,6 +814,7 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
         if (idx.parent().isValid()) {
             QList<QModelIndex> indexes;
             QModelIndex i=idx.parent();
+            QModelIndex p=idx;
             while (i.isValid()) {
                 indexes.prepend(i);
                 i=i.parent();
@@ -825,7 +823,10 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
             listView->setRootIndex(QModelIndex());
             itemModel->setRootIndex(QModelIndex());
             foreach (const QModelIndex &i, indexes) {
-                activateItem(i);
+                activateItem(i, false);
+            }
+            if (p.isValid()) {
+                emit rootIndexSet(p);
             }
             if (scrollTo) {
                 listView->scrollTo(idx, QAbstractItemView::PositionAtTop);
@@ -838,7 +839,7 @@ void ItemView::showIndex(const QModelIndex &idx, bool scrollTo)
 
 void ItemView::focusSearch()
 {
-    if (Mode_Tree==mode || Mode_GroupedTree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode) {
         treeSearch->setFocus();
     } else {
         listSearch->setFocus();
@@ -873,7 +874,7 @@ void ItemView::updateRows(const QModelIndex &idx)
 
 void ItemView::expandAll()
 {
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         treeView->expandAll();
     } else if (Mode_GroupedTree==mode && groupedView) {
         groupedView->expandAll();
@@ -883,25 +884,15 @@ void ItemView::expandAll()
 void ItemView::showSpinner()
 {
     if (!spinner) {
-        #ifdef ENABLE_KDE_SUPPORT
-        spinner=new KPixmapSequenceOverlayPainter(this);
-        spinner->setSequence(KPixmapSequence("process-working", KIconLoader::SizeSmallMedium));
-        #else
-        spinner=new Spinner();
-        #endif
+        spinner=new Spinner(this);
     }
-    spinnerActive=true;
     spinner->setWidget(view()->viewport());
-    #ifdef ENABLE_KDE_SUPPORT
-    spinner->setAlignment(Qt::AlignTop | (Qt::RightToLeft==QApplication::layoutDirection() ? Qt::AlignLeft : Qt::AlignRight));
-    #endif
     spinner->start();
 }
 
 void ItemView::hideSpinner()
 {
     if (spinner) {
-        spinnerActive=false;
         spinner->stop();
     }
 }
@@ -915,12 +906,13 @@ void ItemView::collectionRemoved(quint32 key)
 
 void ItemView::backActivated()
 {
-    if (Mode_Tree==mode || Mode_GroupedTree==mode || 0==currentLevel) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode || 0==currentLevel) {
         return;
     }
     setLevel(currentLevel-1);
     itemModel->setRootIndex(listView->rootIndex().parent());
     listView->setRootIndex(listView->rootIndex().parent());
+    emit rootIndexSet(listView->rootIndex().parent());
 
     if (qobject_cast<QSortFilterProxyModel *>(listView->model())) {
         QModelIndex idx=static_cast<QSortFilterProxyModel *>(listView->model())->mapFromSource(prevTopIndex);
@@ -932,64 +924,26 @@ void ItemView::backActivated()
     }
 }
 
+void ItemView::setExpanded(const QModelIndex &idx, bool exp)
+{
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
+        treeView->setExpanded(idx, exp);
+    }
+}
+
 QAction * ItemView::getAction(const QModelIndex &index)
 {
-    bool rtl = Qt::RightToLeft==QApplication::layoutDirection();
-    bool iconMode=Mode_IconTop==mode && index.child(0, 0).isValid();
-    QRect rect(view()->visualRect(index));
-    rect.moveTo(view()->viewport()->mapToGlobal(QPoint(rect.x(), rect.y())));
-    bool showCapacity = !index.data(ItemView::Role_CapacityText).toString().isEmpty();
-    bool haveToggle = toggle && !index.data(ItemView::Role_ToggleIcon).value<QIcon>().isNull();
-    if (Mode_Tree!=mode || showCapacity) {
-        if (iconMode) {
-            rect.adjust(ActionItemDelegate::constBorder, ActionItemDelegate::constBorder, -ActionItemDelegate::constBorder, -ActionItemDelegate::constBorder);
-        } else {
-            rect.adjust(ActionItemDelegate::constBorder+3, 0, -(ActionItemDelegate::constBorder+3), 0);
-        }
-    }
-
-    if (showCapacity) {
-        int textHeight=QFontMetrics(QApplication::font()).height();
-        rect.adjust(0, 0, 0, -(textHeight+8));
-    }
-
-    QRect actionRect=ActionItemDelegate::calcActionRect(rtl, iconMode, rect);
-    QRect actionRect2(actionRect);
-    ActionItemDelegate::adjustActionRect(rtl, iconMode, actionRect2);
-
-    if (act1 && actionRect.contains(QCursor::pos())) {
-        return act1;
-    }
-
-    if (act1) {
-        ActionItemDelegate::adjustActionRect(rtl, iconMode, actionRect);
-    }
-
-    if (act2 && actionRect.contains(QCursor::pos())) {
-        return act2;
-    }
-
-    if (haveToggle) {
-        if (act1 || act2) {
-            ActionItemDelegate::adjustActionRect(rtl, iconMode, actionRect);
-        }
-
-        if (toggle && actionRect.contains(QCursor::pos())) {
-            return toggle;
-        }
-    }
-
-    return 0;
+    QAbstractItemDelegate *abs=view()->itemDelegate();
+    ActionItemDelegate *d=abs ? qobject_cast<ActionItemDelegate *>(abs) : 0;
+    return d ? d->getAction(index) : 0;
 }
 
 void ItemView::itemClicked(const QModelIndex &index)
 {
-    if ((act1 || act2 || toggle) && ActionItemDelegate::hasActions(index, actLevel)) {
-        QAction *act=getAction(index);
-        if (act) {
-            act->trigger();
-            return;
-        }
+    QAction *act=getAction(index);
+    if (act) {
+        act->trigger();
+        return;
     }
 
     if (forceSingleClick) {
@@ -1004,13 +958,13 @@ void ItemView::itemActivated(const QModelIndex &index)
     }
 }
 
-void ItemView::activateItem(const QModelIndex &index)
+void ItemView::activateItem(const QModelIndex &index, bool emitRootSet)
 {
-    if ((act1 || act2 || toggle) && ActionItemDelegate::hasActions(index, actLevel) && getAction(index)) {
+    if (getAction(index)) {
         return;
     }
 
-    if (Mode_Tree==mode) {
+    if (Mode_SimpleTree==mode || Mode_DetailedTree==mode) {
         treeView->setExpanded(index, !treeView->isExpanded(index));
     } else if (Mode_GroupedTree==mode) {
         if (!index.parent().isValid()) {
@@ -1029,13 +983,16 @@ void ItemView::activateItem(const QModelIndex &index)
         listSearch->setPlaceholderText(i18n("Search %1...").arg(text));
         listView->setRootIndex(index);
         itemModel->setRootIndex(index);
+        if (emitRootSet) {
+            emit rootIndexSet(index);
+        }
         listView->scrollToTop();
     }
 }
 
 void ItemView::delaySearchItems()
 {
-    bool isTree=Mode_Tree==mode || Mode_GroupedTree==mode;
+    bool isTree=Mode_SimpleTree==mode || Mode_DetailedTree==mode || Mode_GroupedTree==mode;
     if ((isTree && treeSearch->text().isEmpty()) || (!isTree && listSearch->text().isEmpty())) {
         if (searchTimer) {
             searchTimer->stop();
